/*  constant.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "constant.h"
#include <klocale.h>
#include <math.h>
#include <kdebug.h>

Constant::Constant(const QString &num, const QString &nam) : Map(QString(),QStringList(), nam)
{
  m_isAClassName="Constant";
//  c=new T(num.toDouble());
  m_c=T::newT(num);
}

Constant::Constant(const T &g) : Map()
{
  m_isAClassName="Constant";
  m_c=T::newT(g);
}

Constant::Constant(const double g, const QString &nam) : Map(QString(),QStringList(), nam)
{
  m_isAClassName="Constant";
  m_c=T::newT(g);
}

Constant::Constant(T *c) : Map()
{
  m_isAClassName="Constant";
  if (c!=0L) m_c=c;
    else m_c=T::newT(0.0);
}

Constant::Constant(const Constant &c) : Map(c)
{
  m_isAClassName="Constant";
  m_c=T::newT(*c.m_c);
}

Constant::~Constant()
{
  T::deleteT(*m_c);
}

Map *Constant::copy(void) const
{
  return new Constant(*this);
}

Map *Constant::eval (const QDict <Map> &)
{
  return copy(); 
}

QString Constant::string(void) const
{
#ifdef DEBUGSTRINGS
  return QString("Constant(%1)").arg(m_c->string());
#else
  return m_c->string();
#endif
}

Map *Constant::add(Constant *&c1,  Constant *&c2)
{
  return new Constant ( T::add(c1->t(),c2->t()) );
}

Map *Constant::sub( Constant *&c1,  Constant *&c2)
{
  return new Constant ( T::sub(c1->t(),c2->t()) );
}

Map *Constant::mul( Constant *&c1,  Constant *&c2)
{
  return new Constant ( T::mul(c1->t(),c2->t()) );
}

Map *Constant::div( Constant *&c1,  Constant *&c2)
{
  return new Constant ( T::div(c1->t(),c2->t()) );
}

Map *Constant::pow(Constant * &m1, Constant * &m2, bool managem1, bool managem2)
{
  Constant *m=new Constant( ::pow((double)*m1->t(),(double)*m2->t()) );
  if (managem1) { delete m1; m1=0L; }
  if (managem2) { delete m2; m2=0L; }
  return m;
}

QString Constant::help(void) const
{
  kdDebug() << helpPrefix() << endl;
  if (!m_name.isEmpty())
    return helpPrefix()+QString("<H3><I>%1 = %2</I></H3><P>User Constant<P>")
//	"<H5><A HREF=\"math://RequestPlotFunction/%3\">Plot Function ...</A></H5>")
      .arg(m_name).arg(string())+helpPostfix();
  return helpPrefix()+QString(i18n("No help available"))+helpPostfix();
} 

Constant *Constant::newConstantG(const T &g)
{
  Constant *c=new Constant(g);
  Garbage::throwMap(c);
  return c;
}

Map *Constant::derive(const Variable &, const QDict <Map> &)
{
  return new Constant(0.0); 
} 
