/*  datapolartorect.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "datapolartorect.h"
#include "../constant.h"
#include "../parameter.h"
#include "../tmatrix.h"
#include "../matrix.h"
#include <kdebug.h>
#include <klocale.h>

DataPolarToRect::DataPolarToRect (void) : 
			MapInternal("",QStringList(),"")
{ 
  m_nParameters=2;
  m_parameters=new Parameter *[m_nParameters];
   
  m_parameters[0]=new Parameter("u");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("v");
  m_parameters[1]->setDataType("Matrix");
}

DataPolarToRect::DataPolarToRect(const DataPolarToRect &m) : MapInternal(m)
{ }

Map *DataPolarToRect::copy(void) const
{ 
  return new DataPolarToRect(*this); 
}

Map *DataPolarToRect::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *mu=vars2->find("u");
   Map *mv=vars2->find("v");
   // XXX Check the matrix dimensions !
   Map *r=0L;

   if (mu->isA("Matrix")&&(mv->isA("Matrix")))
   {
     kdDebug() << "DataPolarToRect::eval" << endl;

     TMatrix *u=mu->tmatrix();
     TMatrix *v=mv->tmatrix();

     TMatrix *tr=new TMatrix(1,3);

     TMatrix *tx=new TMatrix(1,u->nCols());
     TMatrix *ty=new TMatrix(1,u->nCols());
     TMatrix *tz=new TMatrix(1,u->nCols());
     T &su=*T::newT(0.0);
     T &sv=*T::newT(0.0);
     T &cu=*T::newT(0.0);
     T &cv=*T::newT(0.0);

     for (unsigned int i=0;i<u->nCols();i++)
     {
       su=*u->t(0,i);
       cu=su;
       su.sin();
       cu.cos();
       sv=*v->t(0,i);
       cv=sv;
       sv.sin();
       cv.cos();

       tx->set(0,i,new Constant(T::mul(&cu,&sv)));
       ty->set(0,i,new Constant(T::mul(&su,&sv)));
       tz->set(0,i,new Constant(cv));
       
     }
     tr->set(0,0,new Matrix(tx)); 
     tr->set(0,1,new Matrix(ty)); 
     tr->set(0,2,new Matrix(tz)); 

     T::deleteT(su);
     T::deleteT(cu);
     T::deleteT(sv);
     T::deleteT(cv);
     
     r=new Matrix(tr); //XXX Possible mem leakage
   }
   else
   {
     r=new DataPolarToRect;
     r->setParameterExpression(0,mu->copy());
     r->setParameterExpression(1,mv->copy());
   }
   delete vars2;
   return r;
}

QString DataPolarToRect::string(void) const
{
  return QString("DataPolarToRect(%1,%2)").arg(m_parameters[0]->string()).arg(m_parameters[1]->string());
}

QString DataPolarToRect::help(void) const
{
  return helpPrefix()+i18n("<H3><I>DataPolarToRect( matrixPhi, matrixPsi )</I></H3>"
      "<P>Converts a set of (phi,psi) values representing spherical"
      " coordinates to a set of (x,y,z) values representing the respective"
      " cartesian coordinates"
      "<P>"
      "The real transformation for each element of matrixPhi/matrixPsi is :"
      "<P><CENTER>"
      "<BR>x = Cos(phi)*Sin(psi)"
      "<BR>y = Sin(phi)*Sin(psi)"
      "<BR>z = Cos(psi)"
      "</CENTER><P>"
      "<B><I>Examples:</I></B>"
      "DataPolarToRect( [0,PI/2,PI] , [0,0,PI/2] )")+helpPostfix();
}
