/*  exp.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "exp.h"
#include "../constant.h"
#include "../parameter.h"
#include "../operator.h"
#include <kdebug.h>
#include <math.h>

Exp::Exp (const QString &) : MapInternal("","x","")
{ 
  m_isAClassName="Exp";
}

Exp::Exp(const Exp &m) : MapInternal(m)
{ 
  m_isAClassName="Exp";
}

Map *Exp::copy(void) const
{ 
//  return new Sin(""); 
  return new Exp(*this); 
}

Map *Exp::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   Map *r;
   if (x->isA("Constant"))
   {
//     r=new Constant(sqrt((double)*x->t()));
     T *t=x->t()->copy();
     t->exp();
     r=new Constant(t);
   }
   else
   {
     r=new Exp;
     r->setParameterExpression(0,x->copy());  // XXX Is it safe to use x instead
                                             // of x->copy() ?
   }
   delete vars2;
   return r;
}

Map *Exp::derive(const Variable &var, const QDict <Map> &vars)
{
  Map *r=new Exp; /// this ? or empty ?
  r->setParameterExpression(0,m_parameters[0]->copy());
  Map *d=m_parameters[0]->map()->derive(var, vars);
  r=Map::mul(r,d,true,true);
  return r;
}

QString Exp::string(void) const
{
  return QString("Exp(%1)").arg(m_parameters[0]->string());
}

Log::Log (const QString &) :
			MapVariableArgumentsInternal(QString::null,QString::null)
{
  m_isAClassName="Log";

  m_parameters[0]=new Parameter("x");
  m_parameters[0]->setDataType("Constant");
  m_parameters[1]=new Parameter("base");
  m_parameters[1]->setDataType("Constant");
}

Log::Log(const Log &m) : MapVariableArgumentsInternal(m)
{ 
  m_isAClassName="Log";
}

Map *Log::copy(void) const
{ 
//  return new Sin(""); 
  return new Log(*this); 
}

Map *Log::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("$0");
   Map *b=vars2->find("$1");
   Map *r;
   if ((nArgs()==1)&&(x->isA("Constant")))
   {
     //   r=new Constant(sqrt((double)*x->t()));
     T *t=x->t()->copy();
     t->log();
     r=new Constant(t);
   }
   else if ((nArgs()==2)&&(b->isA("Constant"))&&(x->isA("Constant")))
   {
     T *t=x->t()->copy();
     t->log(*b->t());
     r=new Constant(t);
   }
   else
   {
     r=new Log;
     r->setParameterExpression(0,x->copy());  // XXX Is it safe to use x instead
                                             // of x->copy() ?
     if (nArgs()==2)
       r->setParameterExpression(1,b->copy());  
   }
   delete vars2;
   return r;
}

Map *Log::derive(const Variable &var, const QDict <Map> &vars)
{
//  Map *r=new Operator('/', new Constant(1.0), params[0]->map()->copy()); /// this ? or empty ?
//  Map *d=params[0]->map()->derive(var, vars);
//  r=Map::mul(r,d,true,true);
  Map *r=Map::div2(m_parameters[0]->map()->derive(var, vars),m_parameters[0]->map(),true,false);
  return r;
}


QString Log::string(void) const
{
  if (nArgs()==1)
    return QString("Log(%1)").arg(m_parameters[0]->string());
  else if (nArgs()==2)
    return QString("Log(%1,%2)").arg(m_parameters[0]->string()).arg(m_parameters[1]->string()); 
  else return QString("Log");
}
