/*  factorial.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "factorial.h"
#include "../constant.h"
#include "../parameter.h"
#include "../garbage.h"
#include <kdebug.h>
#include <math.h>

Factorial::Factorial () : MapInternal("","x","")
{ }

Factorial::Factorial(const Factorial &m) : MapInternal(m)
{ }

Map *Factorial::copy(void) const
{ 
  return new Factorial(*this); 
}

Map *Factorial::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  Map *x=vars2->find("x");
  Map *r;
  if (x->isA("Constant"))
  {
    T &gr=*T::newT(*x->t());
    T &one=*T::newT(1.0);
    T &g=*T::newT(gr-one);
    for (;g>one;--g) gr*=g; 

    r=new Constant(gr);
    T::deleteT(gr);
    T::deleteT(one);
    T::deleteT(g);
  }
  else
   {
     r=new Factorial;
     r->setParameterExpression(0,x->copy());  
   }
   delete vars2;

   return r;
}

QString Factorial::string(void) const
{
  if ( m_parameters[0]->map() && m_parameters[0]->map()->isA("Operator") )
    return QString("(%1)!").arg(m_parameters[0]->string());

  return QString("%1!").arg(m_parameters[0]->string());
}


