/*  hyp.cpp - 
    This file is part of Kalamaris
    Modified from trig.c by James Tappin
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "hyp.h"
#include "../constant.h"
#include "../parameter.h"
#include <kdebug.h>
#include <math.h>

Sinh::Sinh() : MapInternal("","x","")
{ }

Sinh::Sinh(const Sinh &m) : MapInternal(m)
{ }

Map *Sinh::copy(void) const
{ 
  return new Sinh(*this); 
}

Map *Sinh::eval(const QDict <Map> &vars)
{
//  kdDebug() << "Sinh::eval" << endl;
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   if (!x) return new Sinh;
   Map *r;
   if (x->isA("Constant"))
   {
     r=new Constant(sinh(*((Constant *)x)->t()));
   }
   else
   {
     r=new Sinh;
     r->setParameterExpression(0,x->copy());
   }
   delete vars2;
   return r;
}

Map *Sinh::derive(const Variable &var, const QDict <Map> &vars)
{
  Map *r=new Cosh; /// this ? or empty ?
  r->setParameterExpression(0,m_parameters[0]->copy());
  Map *d=m_parameters[0]->map()->derive(var, vars);
  r=Map::mul(r,d,true,true);
  return r;
}

QString Sinh::string(void) const
{
  return QString("Sinh(%1)").arg(m_parameters[0]->string());
}

Cosh::Cosh () : MapInternal("","x","")
{ }

Cosh::Cosh(const Cosh &m) : MapInternal(m)
{ }
/*
Cosh::~Cosh()
{ 
  kdDebug() << "Cosh::~Cosh" << endl;
}*/

Map *Cosh::copy(void) const
{ 
// kdDebug() << "Cosh::copy()" << endl;
  return new Cosh(*this); 
}

Map *Cosh::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   if (!x) return 0L;
   Map *r;
//   kdDebug() << "aa" << endl;
   if (x->isA("Constant"))
   {
     r=new Constant(cosh(*x->t()));
   }
   else
   {
     r=new Cosh;
     r->setParameterExpression(0,x->copy());
   }
   delete vars2;
   return r;
}

Map *Cosh::derive(const Variable &var, const QDict <Map> &vars)
{
  Map *r=new Sinh;
  r->setParameterExpression(0,m_parameters[0]->copy());
  Map *d=m_parameters[0]->map()->derive(var, vars);
  r=Map::mul(r,d,true,true);
  return r;
}

QString Cosh::string(void) const
{
  return QString("Cosh(%1)").arg(m_parameters[0]->string());
}

Tanh::Tanh () : MapInternal("","x","")
{ }

Tanh::Tanh(const Tanh &m) : MapInternal(m)
{ }

Map *Tanh::copy(void) const
{ 
  return new Tanh(*this); 
}

Map *Tanh::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  Map *x=vars2->find("x");
  if (!x) return 0L;
  Map *r;
  if (x->isA("Constant"))
  {
    r=new Constant(tanh(*((Constant *)x)->t()));
  }
  else
  {
    r=new Tanh;
    r->setParameterExpression(0,x->copy());
  }
  delete vars2;
  return r;
}

Map *Tanh::derive(const Variable &var, const QDict <Map> &vars)
{
  kdDebug() << "Tanh::derive" << endl;
  Map *r=new Cosh;
  r->setParameterExpression(0,m_parameters[0]->copy());
  r=Map::div2(new Constant(1.0),
      		  Map::pow2(r,new Constant(2.0),true,true),
		  true,true); /// this ? or empty ?
  kdDebug() << "Tanh::derive2" << endl;
  kdDebug() << "Tanh::derive3" << endl;
  Map *d=m_parameters[0]->map()->derive(var,vars);
  kdDebug() << "Tanh::derive4" << endl;
  r=Map::mul(r,d,true,true);
  return r;
}

QString Tanh::string(void) const
{
  return QString("Tanh(%1)").arg(m_parameters[0]->string());
}

