/*  sqrt.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "sqrt.h"
#include "../constant.h"
#include "../parameter.h"
#include <kdebug.h>
#include <math.h>

Sqrt::Sqrt () : MapInternal("","x","")
{ }

Sqrt::Sqrt(const Sqrt &m) : MapInternal(m)
{ }

Map *Sqrt::copy(void) const
{ 
  return new Sqrt(*this); 
}

Map *Sqrt::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   Map *r;
   if (x->isA("Constant"))
   {
     T *t=x->t()->copy();
     t->sqrt();
     r=new Constant(t);
   }
   else
   {
     r=new Sqrt;
     r->setParameterExpression(0,x->copy());  // XXX Is it safe to use x instead
                                             // of x->copy() ?
   }
   delete vars2;
   return r;
}

QString Sqrt::string(void) const
{
  return QString("Sqrt(%1)").arg(m_parameters[0]->string());
}


