/*  trig.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "trig.h"
#include "../constant.h"
#include "../parameter.h"
#include <kdebug.h>
#include <math.h>

Sin::Sin() : MapInternal("","x","")
{ }

Sin::Sin(const Sin &m) : MapInternal(m)
{ }

Map *Sin::copy(void) const
{ 
  return new Sin(*this); 
}

Map *Sin::eval(const QDict <Map> &vars)
{
//  kdDebug() << "Sin::eval" << endl;
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   if (!x) return new Sin;
   Map *r;
   if (x->isA("Constant"))
   {
     r=new Constant(sin(*((Constant *)x)->t()));
   }
   else
   {
     r=new Sin;
     r->setParameterExpression(0,x->copy());
   }
   delete vars2;
   return r;
}

Map *Sin::derive(const Variable &var, const QDict <Map> &vars)
{
  Map *r=new Cos; /// this ? or empty ?
  r->setParameterExpression(0,m_parameters[0]->copy());
  Map *d=m_parameters[0]->map()->derive(var, vars);
  r=Map::mul(r,d,true,true);
  return r;
}

QString Sin::string(void) const
{
  return QString("Sin(%1)").arg(m_parameters[0]->string());
}

Cos::Cos () : MapInternal("","x","")
{ }

Cos::Cos(const Cos &m) : MapInternal(m)
{ }
/*
Cos::~Cos()
{ 
  kdDebug() << "Cos::~Cos" << endl;
}*/

Map *Cos::copy(void) const
{ 
// kdDebug() << "Cos::copy()" << endl;
  return new Cos(*this); 
}

Map *Cos::eval(const QDict <Map> &vars)
{
   QDict <Map> *vars2=evalParams(vars);
   Map *x=vars2->find("x");
   if (!x) return 0L;
   Map *r;
//   kdDebug() << "aa" << endl;
   if (x->isA("Constant"))
   {
     r=new Constant(cos(*x->t()));
   }
   else
   {
     r=new Cos;
     r->setParameterExpression(0,x->copy());
   }
   delete vars2;
   return r;
}

Map *Cos::derive(const Variable &var, const QDict <Map> &vars)
{
  Map *r=new Sin;
  r->setParameterExpression(0,m_parameters[0]->copy());
  r=Map::mul2(new Constant(-1.0),r,true,true); /// this ? or empty ?
  Map *d=m_parameters[0]->map()->derive(var, vars);
  r=Map::mul(r,d,true,true);
  return r;
}

QString Cos::string(void) const
{
  return QString("Cos(%1)").arg(m_parameters[0]->string());
}

Tan::Tan () : MapInternal("","x","")
{ }

Tan::Tan(const Tan &m) : MapInternal(m)
{ }

Map *Tan::copy(void) const
{ 
  return new Tan(*this); 
}

Map *Tan::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  Map *x=vars2->find("x");
  if (!x) return 0L;
  Map *r;
  if (x->isA("Constant"))
  {
    r=new Constant(tan(*((Constant *)x)->t()));
  }
  else
  {
    r=new Tan;
    r->setParameterExpression(0,x->copy());
  }
  delete vars2;
  return r;
}

Map *Tan::derive(const Variable &var, const QDict <Map> &vars)
{
  kdDebug() << "Tan::derive" << endl;
  Map *r=new Cos;
  r->setParameterExpression(0,m_parameters[0]->copy());
  r=Map::div2(new Constant(1.0),
      		  Map::pow2(r,new Constant(2.0),true,true),
		  true,true); /// this ? or empty ?
  kdDebug() << "Tan::derive2" << endl;
  kdDebug() << "Tan::derive3" << endl;
  Map *d=m_parameters[0]->map()->derive(var,vars);
  kdDebug() << "Tan::derive4" << endl;
  r=Map::mul(r,d,true,true);
  return r;
}

QString Tan::string(void) const
{
  return QString("Tan(%1)").arg(m_parameters[0]->string());
}

