/*  animatedata.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "animatedata.h"
#include "../constant.h"
#include "../parameter.h"
//#include "../qtai/qtai_plot.h"
#include "../qtai/qtai_anim.h"
#include "graphs.h"
#include <kdebug.h>

AnimateData::AnimateData() : MapVariableArgumentsInternal(QString::null,"AnimateData",false)
{ 
  // XXX
  m_parameters[0]=new Parameter("datax");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("datay");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("graphindex");
  m_parameters[2]->setDataType("Constant");
}

AnimateData::AnimateData(const AnimateData &m) : MapVariableArgumentsInternal( m )
{
}
  
Map *AnimateData::copy(void) const
{
  return new AnimateData(*this);
}

Map *AnimateData::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  unsigned int i;

  double xmin=0,xmax=1,ymin=0,ymax=1;
  TMatrix *xmat=vars2->find("$0")->tmatrix();
  TMatrix *ymat=vars2->find("$1")->tmatrix();

  unsigned int count=QMIN(xmat->nCols(),ymat->nCols());

  double *datax=new double[count];
  for (i=0;i<count;i++) datax[i]=*xmat->at(0,i)->t();
  double *datay=new double[count];
  for (i=0;i<count;i++) datay[i]=*ymat->at(0,i)->t();

  xmin=xmax=datax[0];
  ymin=ymax=datay[0];
  for (i=0;i<count;i++) 
  {
    if (datax[i]<xmin) xmin=datax[i];
    else if (datax[i]>xmax) xmax=datax[i];

    if (datay[i]<ymin) ymin=datay[i];
    else if (datay[i]>ymax) ymax=datay[i];
  }

  if (xmin==xmax)
  {
    xmin-=1;
    xmax+=1;
  }
  if (ymin==ymax)
  {
    ymin-=1;
    ymax+=1;
  }

  int id=0;
  if (nArgs()==3) // We want to insert the graph on another animate widget which
    // is already created
  {
    T *idx=vars2->find("$2")->t();
    id=*idx;

    kdDebug() << "Merge animations" << id << endl;

    AIPlot *plot=Graphs::plot2D(id);
    if (plot)
    {
      plot->setData(plot->traces.count(),count,datax,datay);
      xmin=QMIN(xmin, plot->xmin);
      xmax=QMAX(xmax, plot->xmax);
      ymin=QMIN(ymin, plot->ymin);
      ymax=QMAX(ymax, plot->ymax);
      plot->setRanges(xmin,xmax,ymin,ymax);
    } else id=-1;
  }
  else
  {
    AIPlot *plot=new AIAnim( 0L, "AnimPlotter", count, 1);
    plot->setData(0,count,datax,datay);
    plot->setRanges(xmin,xmax,ymin,ymax);
  //  plot->setPlotColor(1,127,127,0);
    plot->setPlotColor(0,255,0,0);
    plot->setDoubleBuffer(TRUE);
    plot->setPlotType( AI_PLOT_TREND );
    plot->resize(200,200);
    plot->show();

    id=Graphs::add(plot);
  }
  
  delete vars2;
  
  return new Constant(id);
  
};

QString AnimateData::string(void) const
{
  return QString("AnimateData");
}
  
