/*  derive.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "derive.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include <kdebug.h>

Derive::Derive(void) : MapInternal(QString::null,QStringList(),"D")
{
  m_nParameters=2;
  m_parameters=new Parameter *[m_nParameters];

  //params=parms;

  m_parameters[0]=new Parameter("f");
  m_parameters[1]=new Parameter("x");
  m_parameters[1]->setDataType("Variable");
}

Derive::Derive(Map *m, Variable *x) : MapInternal(QString::null,QStringList(),"D")
{
  m_nParameters=2;
  m_parameters=new Parameter *[m_nParameters];

  //params=parms;

  m_parameters[0]=new Parameter("f");
  m_parameters[1]=new Parameter("x");
  m_parameters[1]->setDataType("Variable");

  setParameterExpression(0,m);
  setParameterExpression(1,reinterpret_cast<Map *>(x));
}

Derive::Derive(const Derive &m) : MapInternal(m)
{
}

Derive::~Derive()
{
}
  
Map *Derive::copy(void) const
{
  return new Derive(*this);
}
  
Map *Derive::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  Map *f=vars2->find("f");
  Variable *x=reinterpret_cast<Variable *>(vars2->find("x"));

  Map *r=f->derive(*x, vars);
  
  delete vars2;
  return r;
}

QString Derive::string(void) const
{
  return "D";
}

Map *Derive::simplify(void)
{
  QDict <Map> emptydict;
  return eval( emptydict );
} 
