/*  evalfunc.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "evalfunc.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include <kdebug.h>

EvalFunc::EvalFunc(void) : MapInternal(QString::null,QStringList(),"EvalFunc")
{
  m_nParameters=5;
  m_parameters=new Parameter *[m_nParameters];

  //params=parms;

  m_parameters[0]=new Parameter("f");
  m_parameters[1]=new Parameter("x");
  m_parameters[1]->setDataType("Variable");
  m_parameters[2]=new Parameter("a");
  m_parameters[2]->setDataType("Constant");
  m_parameters[3]=new Parameter("b");
  m_parameters[3]->setDataType("Constant");
  m_parameters[4]=new Parameter("m");
  m_parameters[4]->setDataType("Constant");
}

EvalFunc::EvalFunc(const EvalFunc &m) : MapInternal(m)
{
}

EvalFunc::~EvalFunc()
{
}
  
Map *EvalFunc::copy(void) const
{
  return new EvalFunc(*this);
}
  
Map *EvalFunc::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  Map *f=vars2->find("f");
  Map *xvar=vars2->find("x");
  T &a=*T::newT(*vars2->find("a")->t());
  T &b=*T::newT(*vars2->find("b")->t());
  T &n=*T::newT(*vars2->find("m")->t());
  Map *tx;
  QDict <Map> tmpvars;

  TMatrix *m=new TMatrix(2,(double)n);
  double x=a;
  int i=0;
  T &h=*T::newT(b);
  h-=a;
  h/=(n-1);
  f->propagateForceEvaluation();

  while (i<(double)n)
  {
//    kdDebug() << "i=" << i << " (" << m->nCols() << "), x=" << x << "\n" << endl;
    m->set(0,i,tx=new Constant(x));
    tmpvars.replace(xvar->name(),tx);
    m->set(1,i++,f->eval(tmpvars));
//    x=(double)(a+(b-a)*(T)i/n);
    x+=(double)h;
  }
  
  delete vars2;
  return new Matrix(m);
}

QString EvalFunc::string(void) const
{
  return "EvalFunc";
}
