/*  plotdata.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "plotdata.h"
#include "../constant.h"
#include "../parameter.h"
//#include "../qtai/qtai_plot.h"
#include "../qtai/qtai_anim.h"
#include "graphs.h"
#include <kdebug.h>
#include <math.h>

PlotData::PlotData() : MapVariableArgumentsInternal(QString::null,"PlotData",false)
{ 
  // XXX
  m_parameters[0]=new Parameter("datax");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("datay");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("graphindex");
  m_parameters[2]->setDataType("Constant");
}

PlotData::PlotData(const PlotData &m) : MapVariableArgumentsInternal( m )
{
}
  
Map *PlotData::copy(void) const
{
  return new PlotData(*this);
}

Map *PlotData::eval(const QDict <Map> &vars)
{
  kdDebug() << "PlotData::eval" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  unsigned int i;
/*  for (i=0;i<nArgs();i++)
  {
    kdDebug() << QString("%1) (%2)").arg(i).arg(argument(i)->string()) << endl;
  }
*/
  double xmin=0,xmax=1,ymin=0,ymax=1;
  TMatrix *xmat=vars2->find("$0")->tmatrix();
  TMatrix *ymat=vars2->find("$1")->tmatrix();

  unsigned int count=QMIN(xmat->nCols(),ymat->nCols());
  double *datax=new double[count];
  for (i=0;i<count;i++) datax[i]=*xmat->at(0,i)->t();
  double *datay=new double[count];
  for (i=0;i<count;i++) datay[i]=*ymat->at(0,i)->t();

  xmin=xmax=datax[0];
  for (i=0;i<xmat->nCols();i++) 
  {
    if (datax[i]<xmin) xmin=datax[i];
    else if (datax[i]>xmax) xmax=datax[i];
  }
  ymin=ymax=datay[0];
  for (i=0;i<ymat->nCols();i++) 
  {
    if (datay[i]<ymin) ymin=datay[i];
    else if (datay[i]>ymax) ymax=datay[i];
  }

  if (fabs(xmin-xmax)<0.00001)
  {
    xmin-=1;
    xmax+=1;
  }
  if (fabs(ymin-ymax)<0.00001)
  {
    ymin-=1;
    ymax+=1;
  }
  
  int id=0;
  if (nArgs()==3) // We want to insert the graph on another plot widget which
    // is already created
  {
    kdDebug() << "Merge plots" << endl;
    T *idx=vars2->find("$2")->t();  
    id=*idx;

    AIPlot *plot=Graphs::plot2D(id);
    if (plot) 
    {
    kdDebug() << "Merge plots2" << endl;
      plot->setData(plot->traces.count(),count,datax,datay);
    kdDebug() << "Merge plots3" << endl;
      xmin=QMIN(xmin, plot->xmin);
      xmax=QMAX(xmax, plot->xmax);
      ymin=QMIN(ymin, plot->ymin);
      ymax=QMAX(ymax, plot->ymax);
      plot->setRanges(xmin,xmax,ymin,ymax);
    } else id=-1;
  }
  else
  {
    AIPlot *plot=new AIPlot(0L,"Plotter",count,1);
    plot->setData(0,count,datax,datay);
    plot->setRanges(xmin,xmax,ymin,ymax);
//    plot->setPlotColor(1,127,127,0);
    plot->setPlotColor(0,255,0,0);
    plot->setDoubleBuffer(TRUE);
    plot->setPlotType( AI_PLOT_TREND );
    plot->resize(200,200);
    plot->show();

    id=Graphs::add(plot);
  }

  delete vars2;

  return new Constant(id);
}

QString PlotData::string(void) const
{
  return QString("PlotData");
}
