/*  plotdata3d.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2001  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "plotdata3d.h"
#include "../constant.h"
#include "../parameter.h"
//#include "../qtai/qtai_plot.h"
#include "../kgl/kgldataanimate.h"
#include "../kgl/kgl3dwidget.h"
#include "../kgl/kglcontrol.h"
#include "graphs.h"
#include <kdebug.h>

PlotData3D::PlotData3D() : MapVariableArgumentsInternal(QString::null,"PlotData3D",false)
{ 
  // XXX
  m_parameters[0]=new Parameter("datax");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("datay");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("dataz");
  m_parameters[2]->setDataType("Matrix");
}

PlotData3D::PlotData3D(const PlotData3D &m) : MapVariableArgumentsInternal( m )
{
}
  
Map *PlotData3D::copy(void) const
{
  return new PlotData3D(*this);
}

Map *PlotData3D::eval(const QDict <Map> &vars)
{
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;

  unsigned int i;
  TMatrix *xmat=vars2->find("$0")->tmatrix();
  TMatrix *ymat=vars2->find("$1")->tmatrix();
  TMatrix *zmat=vars2->find("$2")->tmatrix();

  unsigned int count=QMIN(xmat->nCols(),ymat->nCols());
  count=QMIN(count,zmat->nCols());
  
  double *datax=new double[count];
  for (i=0;i<count;i++) datax[i]=*xmat->t(0,i);
  double *datay=new double[count];
  for (i=0;i<count;i++) datay[i]=*ymat->t(0,i);
  double *dataz=new double[count];
  for (i=0;i<count;i++) dataz[i]=*zmat->t(0,i);

  int id;

  if (nArgs()==4) // We want to insert the graph on another plot widget which
    // is already created
  {
    kdDebug() << "Merge plots" << endl;
    T *idx=vars2->find("$3")->t();
    id=*idx;

    KGL3DWidget *kgl3dwidget=Graphs::plot3D(id);
    if (kgl3dwidget) 
    {
      // XXX Explicit cast could be wrong !!!
      KGLDataAnimate *plot=reinterpret_cast<KGLDataAnimate *>(kgl3dwidget->universe()); 
      int trace = plot->plotData(datax,datay,dataz,count);
      plot->setPlotColor( trace, Qt::red );
    } else id=-1;
  }
  else
  {
    KGL3DWidget *kgl3dwidget=new KGL3DWidget(0L,"3dview");
    KGLDataAnimate *plot=new KGLDataAnimate(kgl3dwidget,"AnimPlotter3D");
    kgl3dwidget->setUniverse(plot);
    kgl3dwidget->controls()->show();
    int trace=plot->plotData(datax,datay,dataz,xmat->nCols());
    plot->setPlotColor( trace, Qt::red );
  //  kgl3dwidget->resize(400,400);
    kdDebug() << "aa\n";
    kgl3dwidget->show();
    // plot->show();

    id=Graphs::add(kgl3dwidget);
  }

  delete vars2;
  return new Constant(id);
}

QString PlotData3D::string(void) const
{
  return QString("PlotData3D");
}
