/*  plotdirectionfield3d.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2001  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "plotdirectionfield3d.h"
#include "plotorbit.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include "../kgl/kglline.h"
#include "../kgl/kgluniverse.h"
#include "../kgl/kgldataanimate.h"
#include "../kgl/kgl3dwidget.h"
#include "../kgl/kglcontrol.h"
#include "graphs.h"
#include <qstring.h>
#include <kdebug.h>
#include <kapp.h>
#include <unistd.h>
#include <math.h>

PlotDirectionField3D::PlotDirectionField3D(void) : MapVariableArgumentsInternal(QString::null,"PlotDirectionField3D",false)
{
  m_nParameters=4;
  m_parameters=new Parameter *[m_nParameters];

  m_parameters[0]=new Parameter("f");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("backbottomleftcorner");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("foreupperrightcorner");
  m_parameters[2]->setDataType("Matrix");
  m_parameters[3]=new Parameter("partition");
  m_parameters[3]->setDataType("Matrix");
//  m_parameters[4]=new Parameter("graphindex");
//  m_parameters[4]->setDataType("Constant");
}

PlotDirectionField3D::PlotDirectionField3D(const PlotDirectionField3D &m) : MapVariableArgumentsInternal(m)
{
}

PlotDirectionField3D::~PlotDirectionField3D()
{
}
  
Map *PlotDirectionField3D::copy(void) const
{
  return new PlotDirectionField3D(*this);
}

Map *PlotDirectionField3D::eval(const QDict <Map> &vars)
{
  kdDebug() << "PlotDirectionField3D::eval" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;

  TMatrix &f=*(new TMatrix(*vars2->find("$0")->tmatrix()));
  /*int id_f=*/Graphs::add(&f);
  const TMatrix backbottomleft(*vars2->find("$1")->tmatrix());
  const TMatrix foreupperright(*vars2->find("$2")->tmatrix());
  const TMatrix partition(*vars2->find("$3")->tmatrix());                                    
  double xmin=backbottomleft.tRef(0);
  double ymin=backbottomleft.tRef(1);
  double zmin=backbottomleft.tRef(2);
  double xmax=foreupperright.tRef(0);
  double ymax=foreupperright.tRef(1);
  double zmax=foreupperright.tRef(2);
  int xpart=partition.tRef(0);
  int ypart=partition.tRef(1);
  int zpart=partition.tRef(2);

  unsigned int n=f.nRows();
  
  unsigned int i;
  for (i=0;i<n;i++) 
    f.at(i)->propagateForceEvaluation();

  double dx=(xmax-xmin)/xpart;
  double dy=(ymax-ymin)/ypart;
  double dz=(zmax-zmin)/zpart;
  Map *r;

  double *xderiv= new double[xpart*ypart*zpart];
  double *yderiv= new double[xpart*ypart*zpart];
  double *zderiv= new double[xpart*ypart*zpart];

  Garbage::push();
  for (int k=0;k<zpart;k++)
  {
    Garbage::push();
    vars2->replace("y2",Constant::newConstantG(zmin+k*dz));
    for (int i=0;i<ypart;i++)
    {
      vars2->replace("y1",Constant::newConstantG(ymin+i*dy));
      for (int j=0;j<xpart;j++)
      {
	vars2->replace("y0",Constant::newConstantG(xmin+j*dx));

	r=f.at(0)->eval(*vars2);
	xderiv[k*xpart*ypart+i*xpart+j]=*r->t();
	delete r;

	r=f.at(1)->eval(*vars2);
	yderiv[k*xpart*ypart+i*xpart+j]=*r->t();
	delete r;

	r=f.at(2)->eval(*vars2);
	zderiv[k*xpart*ypart+i*xpart+j]=*r->t();
	delete r;
//	      kdDebug() << i << " " << j <<" " << xderiv[k*xpart*ypart+i*xpart+j] << " " << yderiv[k*xpart*ypart+i*xpart+j] << " " << zderiv[k*xpart*ypart+i*xpart+j] << "\n";
      }
    }
    Garbage::pop();
  }
  Garbage::pop();

  int id;
  KGLDataAnimate *plot=0L;
  if (nArgs()==5) // We want to insert the graph on another plot widget which
    // is already created
  {
    T *idx=vars2->find("$4")->t();
    id=*idx;
 
    KGL3DWidget *kgl3dwidget=Graphs::plot3D(id);
    if (kgl3dwidget)
    {
      // XXX Explicit cast could be wrong !!!
      plot=reinterpret_cast<KGLDataAnimate *>(kgl3dwidget->universe());
    } else id=-1;
  }
  else
  {
    KGL3DWidget *kgl3dwidget=new KGL3DWidget(0L,"3dview");
    plot=new KGLDataAnimate(kgl3dwidget,"AnimPlotter3D");
    kgl3dwidget->setUniverse(plot);
    kgl3dwidget->controls()->show();
    kgl3dwidget->show();
 
    id=Graphs::add(kgl3dwidget);
  }

  if (plot)
  {
    KGLObject *dirfield=createGLObject(xmin,ymin,zmin,xmax,ymax,zmax,
	xpart,ypart,zpart,xderiv,yderiv,zderiv);
    dirfield->setColor(Qt::white);
    dirfield->setMaterialLightEmission(Qt::white);
    plot->insertObject(dirfield);
  }
  kdDebug() << "PlotDirectionField3D::eval(end)" << endl;
  delete vars2;
  
  return new Constant(id);
}

KGLObject *PlotDirectionField3D::createGLObject(double xmin,double ymin,double zmin,
    double xmax,double ymax,double zmax, int xpart, int ypart, int zpart,
    double *xderiv, double *yderiv, double *zderiv)
{
  double x,y,z;
  double dx=(xmax-xmin)/xpart;
  double dy=(ymax-ymin)/ypart;
  double dz=(zmax-zmin)/zpart;
  double *ptx=new double[2*xpart*ypart*zpart];
  double *pty=new double[2*xpart*ypart*zpart];
  double *ptz=new double[2*xpart*ypart*zpart];
  int c=0;
  int cderiv=0;
  double l,lmax=0;
  for (int k=0;k<zpart;k++) 
    for (int j=0;j<ypart;j++) 
      for (int i=0;i<xpart;i++) 
      {
	l=sqrt(xderiv[cderiv]*xderiv[cderiv]+
	    yderiv[cderiv]*yderiv[cderiv]+zderiv[cderiv]*zderiv[cderiv]);
	if (l>lmax) lmax=l;
      }

  cderiv=0;
  for (int k=0;k<zpart;k++) 
  {
    z=zmin+k*dz;
    for (int j=0;j<ypart;j++) 
    {
      y=ymin+j*dy;
      for (int i=0;i<xpart;i++) 
      {
	x=xmin+i*dx;
        ptx[c]=x-0.2*xderiv[cderiv]/lmax;
        pty[c]=y-0.2*yderiv[cderiv]/lmax;
        ptz[c]=z-0.2*zderiv[cderiv]/lmax;
        c++;
        ptx[c]=x+0.2*xderiv[cderiv]/lmax;
        pty[c]=y+0.2*yderiv[cderiv]/lmax;
        ptz[c]=z+0.2*zderiv[cderiv]/lmax;
        c++;
        cderiv++;
      }
    }
  }
  KGLObject *r=new KGLLine(ptx,pty,ptz,c,true);
  delete ptx;
  delete pty;
  delete ptz;

  return r;
}

QString PlotDirectionField3D::string(void) const
{
  return "PlotDirectionField3D";
}

QString PlotDirectionField3D::help(void) const
{
  return helpPrefix()+"<H3><I>PlotDirectionField3D( ... )</I></H3>"
    " "+helpPostfix();
}
