/*  plotorbit.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2001  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "plotorbit.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include "../qtai/qtai_plot.h"
#include "graphs.h"
#include <qstring.h>
#include <kdebug.h>
#include <kapp.h>
#include <kstddirs.h>
#include <unistd.h>
#include "../methods/fehlberg78.h"

PlotOrbit::PlotOrbit(void) : MapVariableArgumentsInternal(QString::null,"PlotOrbit",false)
{
  m_nParameters=2;
  m_maxParameters=2;
  m_parameters=new Parameter *[m_nParameters];

  m_parameters[0]=new Parameter("f");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("y0");
  m_parameters[1]->setDataType("Matrix");
}

PlotOrbit::PlotOrbit(const PlotOrbit &m) : MapVariableArgumentsInternal(m)
{
}

PlotOrbit::~PlotOrbit()
{
}
  
Map *PlotOrbit::copy(void) const
{
  return new PlotOrbit(*this);
}


int PlotOrbit::plotOrbit(int id, int id_f, double ptx, double pty)
{
  Garbage::push();
  
  TMatrix &f=*Graphs::diffEq(id_f);
  unsigned int n=f.nRows();
 // n must be 2 !!! XXX
  T &tol=T::newTG(0.0001);
  T &hfactor=T::newTG(1.1);
  int maxIter=15;
  Map *mnumPoints=maps.find("_numPoints");
  unsigned int numPoints=75;
  if (mnumPoints) numPoints=(unsigned int) *mnumPoints->t();
//  int numPoints=(int) *maps.find("_numPoints")->t();
  T &e=*T::newT(0.0);
  bool stepControl=true;
  QDict <Map> tmpvars;
 
  TMatrix x(1,2);
  TMatrix y(n,2);
  TMatrix z(n,1);
  TMatrix H(14,n);
  unsigned int i,k,j;
  
  y.set(0,0,new Constant(ptx));
  y.set(1,0,new Constant(pty));
  for (i=0;i<n;i++)
    f.at(i)->propagateForceEvaluation();
 
  T &h=*T::newT(0.01);  // Seems a nice first step size
  Map *fi;
 
  TMatrix fx(n,13);
  bool increaseH=false;

  x.set(0,0,new Constant(0.0));
 
  T &c1=*T::newT(41.0/840);
  T &c2=*T::newT(34.0/105);
  T &c3=*T::newT(9.0/35);
  T &c4=*T::newT(9.0/280);
  T &cc1=*T::newT(34.0/105);
  T &cc2=*T::newT(9.0/35);
  T &cc3=*T::newT(9.0/280);
  T &cc4=*T::newT(41.0/840);
  T *t=0L;
  kdDebug() << "Fehlberg78::eval3\n" << endl;
 
  TMatrix *coef=new TMatrix(1,1);
  kdDebug() << "load matrix" << endl;
  coef->load(locate("appdata","coefficients/fehlberg78"));
  kdDebug() << "matrix loaded" << endl;
  kdDebug() << coef->string() << endl;
 
  k=0;
  TMatrix *xx=new TMatrix(13,1);

  while (k<numPoints)
  {
    Garbage::push();
    k++;
    kdDebug() << "k ( " << k << " )" << endl;
    kdDebug() << "xk ( " << static_cast<double>(x.tRef(k-1)) << " )" << endl;
 
    bool done=false;
    int it=0;
    while (!done)
    {
      Garbage::push();
 
      xx->set(0,0,x.at(k-1)->copy());
      xx->set(1,0,new Constant(x.tRef(k-1)+T::newTG(2.0/27)*h));
      xx->set(2,0,new Constant(x.tRef(k-1)+T::newTG(1.0/9)*h));
      xx->set(3,0,new Constant(x.tRef(k-1)+T::newTG(1.0/6)*h));
      xx->set(4,0,new Constant(x.tRef(k-1)+T::newTG(5.0/12)*h));
      xx->set(5,0,new Constant(x.tRef(k-1)+T::newTG(1.0/2)*h));
      xx->set(6,0,new Constant(x.tRef(k-1)+T::newTG(5.0/6)*h));
      xx->set(7,0,new Constant(x.tRef(k-1)+T::newTG(1.0/6)*h));
      xx->set(8,0,new Constant(x.tRef(k-1)+T::newTG(2.0/3)*h));
      xx->set(9,0,new Constant(x.tRef(k-1)+T::newTG(1.0/3)*h));
      xx->set(10,0,new Constant(x.tRef(k-1)+h));
      xx->set(11,0,new Constant(x.tRef(k-1)));
      xx->set(12,0,new Constant(x.tRef(k-1)+h));
      for (int l=0;l<12;l++)
	MethodFehlberg78::calcHi(f,x,y,tmpvars,k,n,l,h,H,*xx,fx,*coef);
 
      for (i=0;i<n;i++)
      {
        fi=f.at(i);
 
        /**
         * Now we evaluate f (x   , y     , ..., y     )
         *                  i  k-1   0,k-1        n,k-1
         */
        tmpvars.clear();
 
        tmpvars.replace("x",xx->at(12,0));
        for (j=0;j<n;j++)
          tmpvars.replace( QString("y%1").arg(j) , H.at(11,j) );
 
        fx.set(i,12,fi->eval(tmpvars));
 
        t=T::mul(&h,&(c1*fx.tRef(i,0)+c2*fx.tRef(i,5)+c3*fx.tRef(i,6)+c3*fx.tRef(i,7)+c4*fx.tRef(i,8)+c4*fx.tRef(i,9)+c1*fx.tRef(i,10)));
        y.set(i,k,new Constant(T::add(y.t(i,k-1),t)));
        delete t;
 
        if (stepControl)
        {
          t=T::mul(&h,&(cc1*fx.tRef(i,5)+cc2*fx.tRef(i,6)+cc2*fx.tRef(i,7)+cc3*fx.tRef(i,8)+cc3*fx.tRef(i,9)+cc4*fx.tRef(i,11)+cc4*fx.tRef(i,12)));
          z.set(i,0,new Constant(T::add(y.t(i,k-1),t)));
          delete t;
        }
 
//      delete fx6;
      }
 
      if (stepControl)
      {
        e=TMatrix::distMaxCol(&y,k,&z,0);
 
        it++;
        increaseH=false;
 
        if ((e<=tol)||(it>=maxIter))
        {
          if (it<maxIter) // == if (e<=tol)
          {
            kdDebug() << QString("h = %1").arg(h.string()) << endl;
            increaseH=true;
          }
          else
            kdDebug() << QString("Max iterations exceeded : h = %1").arg(h.string()) << endl;
 
          done=true;
 
        }
        else
        {
          kdDebug() << "Let's do another iteration with a smaller h" << endl;
          h*=((tol/e).sqrt(4));
          h*=T::newTG(0.9);
        }
      }
      else  // the step control is disabled
        done=true;
 
      Garbage::pop();
//      kapp->processEvents(200);
    }
 
    x.set(0,k,new Constant(T::add(x.t(k-1),&h)));
 
    if (increaseH) h*=hfactor;
 
    Garbage::pop();
  }
 
  kdDebug() << "Fehlberg78::eval(end)" << endl;
//  delete vars2;
 
  T::deleteT(c4);
  T::deleteT(c3);
  T::deleteT(c2);
  T::deleteT(c1);
  T::deleteT(cc4);
  T::deleteT(cc3);
  T::deleteT(cc2);
  T::deleteT(cc1);
 
  T::deleteT(e);
  T::deleteT(hfactor);
  T::deleteT(tol);

  Map *m=functionFactory("PlotData");
  m->setParameterExpression(0,new Matrix(y.row(0)));
  m->setParameterExpression(1,new Matrix(y.row(1)));
  if (id>-1) m->setParameterExpression(2,new Constant(id));

  QDict <Map> emptydict;  
  Map *r=m->eval(emptydict);
  id=*r->t();

  AIPlot *plot=Graphs::plot2D(id);
  if (plot)
    plot->setCustomMode("Initial Condition Mode",id, id_f,
	static_cast <int (*)(int,int,double,double)>(&PlotOrbit::plotOrbit));

  Garbage::pop();
  return id;
}


Map *PlotOrbit::eval(const QDict <Map> &vars)
{
  kdDebug() << "PlotOrbit::eval" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;

  TMatrix *f=new TMatrix(*vars2->find("$0")->tmatrix());
  int id_f=Graphs::add(f);  // XXX Mem is leaked with f !!
  const TMatrix y0(*vars2->find("$1")->tmatrix());
  double xic=y0.tRef(0);
  double yic=y0.tRef(1);
  int id=-1;
  if (nArgs()==3) 
    id=*vars2->find("$2")->t();

  id=plotOrbit(id, id_f, xic, yic);
 
  kdDebug() << "Fehlberg78::eval(end2)" << endl;
  return new Constant(id);
}

QString PlotOrbit::string(void) const
{
  return "PlotOrbit";
}

QString PlotOrbit::help(void) const
{
  return helpPrefix()+"<H3><I>PlotOrbit( ... )</I></H3>"
    " "+helpPostfix();
}
