/*  interpreter.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "interpreter.h"
#include <qstring.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qdict.h>
#include <qregexp.h>
#include <kdebug.h>
#include <klocale.h>
#include "map.h"
#include "mapvararg.h"
#include "constant.h"
#include "methods/mapmethod.h"

Interpreter::Interpreter() : QObject(0,"Interpreter"), DCOPObject ( "interpreter" )
{
  seq=0;
  evaluateFunctions=false;
  self=this;
}

Interpreter::~Interpreter()
{
  self=0L;
}

int Interpreter::process(QString input)
{
  if (input.isEmpty()) return ++seq;
  QRegExp regex("[^=]=[^=]");
  int r;

  kdDebug() << QString("Interpreter::process (%1)").arg(input) << endl;
  emit inputString(++seq, input);

  QRegExp commentary("^ *#");
  
  if (input.find(commentary)!=-1) return seq;

  Garbage::push();

  if ((r=input.find(regex))!=-1)
  {  // Definition of a new variable/map
    kdDebug() << "New definition" << endl;
    QString mapnam=input.left(r+1);
    QString mapdef=input.mid(r+2);
    QString mapname;
    kdDebug() << QString("mapnam %1 , mapdef %2\n").arg(mapnam).arg(mapdef) << endl;

    if ((r=mapnam.find("("))!=-1)
    {
      QStringList paramlist;
      mapname=mapnam.left(r);
      paramlist=QStringList::split(",",mapnam.mid(r+1,mapnam.length()-r-1-1));

//       new Map(mapdef,paramlist,mapname);

      Map *m=new Map(mapdef,paramlist,"");
      m->simplifyFunction();
      m->setName(mapname);
      
    }
    else
    {
      mapname=mapnam;
      
      kdDebug() << QString("mapname (%1) (%2)\n").arg(mapname).arg(mapdef) << endl;
      Map *m=Map::parse(mapdef);
      if (!m) 
      {
	Garbage::pop();
	return seq;
      }
      kdDebug() << m->string() << endl;
      Map *r=m->simplify();
      if (r)
      {
	delete m;
	m=r;
      }
      m->setName(mapname);
    }

    emit outputString(seq, QString(i18n("%1 defined as %2")).arg(mapname).arg(mapdef));

    Garbage::pop();
    return seq;
  }

  // Evaluation

  Map *tmpmap=Map::parse(input);
  if (!tmpmap) 
  {
    Garbage::pop();
    return seq; // XXX should emit an error 
  }

  QDict <Map> emptydict;
  if (!evaluateFunctions) 
  {
    emptydict.insert("$$__noEvaluateFunctions",tmpmap);
    if (!tmpmap->name().isEmpty()) 
	tmpmap->setForceEvaluation(true);
    	// With this check,
    	// we allow it to evaluate just the first function. This is so, because
        // even if the user don't want to evaluate the functions, he'd like to
        // see the contents of f(x) when he types "f(x)", isn't it ? :-)
    
  }
  kdDebug() << "Interpreter::evaluating..." << endl;
  Map *t=tmpmap->eval(emptydict);
  QString result=t->string();
  emit outputString(seq, result);
  delete t;
  delete tmpmap;
  
  Garbage::pop();
  
  return seq;

}

void Interpreter::setFlagEvaluateFunctions( bool b )
{
  evaluateFunctions = b;
}

void Interpreter::setStepControl( bool b )
{
//  MapMethod::setStepControl(b);
  process(QString("_stepControl=%1").arg((b)?1:0));
}

QStringList Interpreter::mapNames()
{
  QStringList names;

  for (QDictIterator <Map> it(Map::maps) ; it.current() ; ++it)
    if (names.findIndex(it.currentKey())==-1) 
      names << it.currentKey();
  
  return names; 
}

Interpreter *Interpreter::self;
