/*  kglobject.cpp - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglobject.h"
#include <kdebug.h>
#include <qgl.h>
#include <kgluniverse.h>

GLfloat kgl_white[] = { 1.0, 1.0, 1.0, 0.0 };

KGLObject::KGLObject(GLfloat x, GLfloat y, GLfloat z)
{
  m_pos[0]=x;
  m_pos[1]=y;
  m_pos[2]=z;
  m_rotationAxis[0]=0;
  m_rotationAxis[1]=0;
  m_rotationAxis[2]=0;
  m_rotationAngle=0;
  m_list=0;
  m_color[0]=m_defaultcolor[0];
  m_color[1]=m_defaultcolor[1];
  m_color[2]=m_defaultcolor[2];
  m_color[3]=m_defaultcolor[3];
  m_lightEmission[0]=0.2;
  m_lightEmission[1]=0.2;
  m_lightEmission[2]=0.2;
  m_lightEmission[3]=1.0;
  m_emitsLight=false;
  m_visible=true;
  m_universe=0L;
  m_id=0;
  for (int i=0;i<16;i++) m_rotationMatrix[i]=0;
  m_rotationMatrix[0]=m_rotationMatrix[5]=m_rotationMatrix[10]=
	m_rotationMatrix[15]=1;
}

KGLObject::~KGLObject()
{
  glDeleteLists(m_list,1);
  if (m_universe) m_universe->removeObject(m_id);
  unplug();
}

void KGLObject::plug( KGLUniverse * universe , int id)
{
  if (m_universe)
  {
    kdWarning() << "KGL Object plugged twice !!! \n"; 
    return;
  }
  m_universe=universe;
  m_id=id;
}

void KGLObject::unplug()
{
  m_universe=0L;
}

void KGLObject::paint()
{
  if (!m_visible) return;
  
  glPushMatrix();

  if (!m_customTexture.isNull())
  {
    glEnable( GL_TEXTURE_2D );
    glTexImage2D( GL_TEXTURE_2D, 0, 3, m_customTexture.width(),
	m_customTexture.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE,
	m_customTexture.bits() );
//    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, m_color );
//    glMaterialfv( GL_FRONT, m_material, m_color );
  }

  glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, m_color );
  if (m_emitsLight)  glMaterialfv( GL_FRONT, GL_EMISSION, m_lightEmission );

  glTranslatef(m_pos[0],m_pos[1],m_pos[2]);
  if (m_rotationAngle!=0) glRotatef(m_rotationAngle,
      m_rotationAxis[0],m_rotationAxis[1],m_rotationAxis[2]);
  glMultMatrixf(m_rotationMatrix);
  
  glCallList(m_list);
  if (m_emitsLight)  glMaterialfv( GL_FRONT, GL_EMISSION, c_black );

  glPopMatrix();

  glDisable( GL_TEXTURE_2D );
  if (!m_customTexture.isNull())
  {
//    gluQuadricTexture(q, GL_FALSE);
    glDisable( GL_TEXTURE_2D );
  }
}

void KGLObject::color(const QColor &c, GLfloat alpha)
{
  m_defaultcolor[0]=((GLfloat)c.red())/255.0;
  m_defaultcolor[1]=((GLfloat)c.green())/255.0;
  m_defaultcolor[2]=((GLfloat)c.blue())/255.0;
  m_defaultcolor[3]=alpha;
}

GLfloat KGLObject::m_defaultcolor[4]={ 1.0, 1.0, 1.0, 1.0};
GLfloat KGLObject::m_tempcolor[4]={ 1.0, 1.0, 1.0, 1.0};

GLfloat *KGLObject::glcolor(const QColor &c, GLfloat alpha)
{
  m_tempcolor[0]=((GLfloat)c.red())/255.0;
  m_tempcolor[1]=((GLfloat)c.green())/255.0;
  m_tempcolor[2]=((GLfloat)c.blue())/255.0;
  m_tempcolor[3]=alpha;
  return m_tempcolor;
}

void KGLObject::moveTo(GLfloat x, GLfloat y, GLfloat z)
{
  m_pos[0]=x;
  m_pos[1]=y;
  m_pos[2]=z;
}  

void KGLObject::rotate(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
{
  if (!angle) return;

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadMatrixf(m_rotationMatrix);

  glLoadIdentity();
  glRotatef(angle, x,y,z);
  glMultMatrixf(m_rotationMatrix);
  glGetFloatv(GL_MODELVIEW_MATRIX, m_rotationMatrix);
  glPopMatrix();
}
  
void KGLObject::setColor(const QColor &c, GLfloat alpha)
{
  setQColor(c);
  glcolor(c, alpha);
  m_color[0]=m_tempcolor[0];
  m_color[1]=m_tempcolor[1];
  m_color[2]=m_tempcolor[2];
  m_color[3]=m_tempcolor[3];
}

void KGLObject::setQColor(const QColor &c)
{
  m_qcolor=c;
}

void KGLObject::setID(int id)
{
  m_id=id;
}

void KGLObject::setMaterialLightEmission(const QColor &c, GLfloat alpha)
{
  glcolor(c,alpha);
  m_lightEmission[0]=m_tempcolor[0];
  m_lightEmission[1]=m_tempcolor[1];
  m_lightEmission[2]=m_tempcolor[2];
  m_lightEmission[3]=m_tempcolor[3];
  m_emitsLight=true;
}

void KGLObject::setTexture(const QImage &texture)
{
  if (!texture.isNull())
    m_customTexture = texture.convertDepth( 32 ).mirror().swapRGB();
}

const GLfloat KGLObject::c_black[4]={ 0.0, 0.0, 0.0, 1.0 };
const GLfloat KGLObject::c_white[4]={ 1.0, 1.0, 1.0, 1.0 };
