/*  kglobject.h - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#ifndef __KGLOBJECT_H
#define __KGLOBJECT_H

#include <qgl.h>
#include <qimage.h>
#include <qcolor.h>

class KGLObject
{
friend class KGLUniverse;

public:
  KGLObject(GLfloat x, GLfloat y, GLfloat z);
  virtual ~KGLObject();

  GLuint listID() const { return m_list; };

  static GLfloat *color() { return m_defaultcolor; };
  QColor qcolor() const { return m_qcolor; };
  static void color(const QColor &c, GLfloat alpha=1.0);
  static GLfloat *glcolor(const QColor &c, GLfloat alpha=1.0);

  void moveTo(GLfloat x, GLfloat y, GLfloat z);
  void rotate(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);

  void setColor(const QColor &color, GLfloat alpha=1.0);
  void setQColor(const QColor &color);
  void setTexture(const QImage &texture);
  void setMaterialLightEmission(const QColor &color, GLfloat alpha=1.0);

  virtual void paint();

  void show() { m_visible = true; };
  void hide() { m_visible = false; };
  bool isVisible() const { return m_visible; };

protected:
  GLuint m_list;
  int m_id;

  GLfloat m_pos[3]; 
  GLfloat m_rotationAxis[3]; 
  GLfloat m_rotationAngle; 
  GLfloat m_rotationMatrix[16];
  GLfloat m_color[4];
  QImage m_texture, m_customTexture;
  GLfloat m_lightEmission[4];
  bool m_emitsLight;

  class KGLUniverse *m_universe;

  bool m_visible;
  static const GLfloat c_black[4];
  static const GLfloat c_white[4];

  static GLfloat m_defaultcolor[4];
  static GLfloat m_tempcolor[4];

  QColor m_qcolor;

  void setID(int id);
  void plug( KGLUniverse * universe , int id);
  void unplug();
};
#endif
