/*  kgluniverse.cpp - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kgluniverse.h"
#include <kaction.h>
#include <klocale.h>
#include <kdebug.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <math.h>
#include <stdio.h>
#include <GL/glu.h>
#include "kglcylinder.h"
#include "kgltriangle.h"
#include "kglsphere.h"
#include "kglrectangle.h"
#include "kglarrow.h"
#include "kglaxis.h"

KGLUniverse::KGLUniverse( QWidget* parent, const char* name )
	: QGLWidget (parent,name)
{
  m_isInitialized=false;
  m_paintElements=c_Axis;
  m_changeObserver=0L;
  for (int i=0;i<c_NumObservers;i++)
  {
    m_currentObserver=i;
    m_pos[m_currentObserver][0]=0;
    m_pos[m_currentObserver][1]=0;
    m_pos[m_currentObserver][2]=0;
    m_viewAngle[m_currentObserver]=40.0;
    m_distanceToCenter[m_currentObserver]=0;
    
    lookTo(0.0,0.0,0.0);

    m_dir[m_currentObserver][0]=dir[0];
    m_dir[m_currentObserver][1]=dir[1];
    m_dir[m_currentObserver][2]=dir[2];
    m_obs[m_currentObserver][0]=obs[0];
    m_obs[m_currentObserver][1]=obs[1];
    m_obs[m_currentObserver][2]=obs[2];
  }

  pos[0]=0;
  pos[1]=0;
  pos[2]=0;
  viewAngle=40.0;
  distanceToCenter=0;
  m_speed=0.8;

  m_currentObserver=0;
  m_staticObserver=false;
  m_customObserver=false;
  m_customObserverMethod=0L;
  m_customRelX=0;
  m_customObserverAllowRelativeMovement=true;
  lookTo(0.0,0.0,0.0);

  disableUpdates=false;
  prevTwist=0;

  objects=new QList<KGLObject>();
  mouseLMBAction=ac_rotateLookDirection;
  mouseMMBAction=ac_rotatePointOfView;

  m_delay= new QTimer(this);
  connect(m_delay,SIGNAL(timeout()), this, SLOT(hideRotationCenter()));
}

KGLUniverse::~KGLUniverse()
{
// XXX delete everything !
  objects->setAutoDelete(false);
  for (KGLObject *it=objects->first();it!=0L;it=objects->next())
    delete it;
  delete objects;
  delete m_changeObserver;
}

static float front_shininess[] =   { 60.0};
static float front_specular[]  =   { 0.7, 0.7, 0.7, 1.0 };
 
void KGLUniverse::initializeGL()
{
  // Set up the lights

  GLfloat white[4] = {2.0, 2.0, 2.0, 1.0};
  GLfloat lightDir[3] = {-1.0, -1.0, 0.0};

  glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//  glBlendFunc(GL_SRC_ALPHA_SATURATE, GL_ONE);
//  glEnable(GL_POLYGON_SMOOTH);
  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

  glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
  glMaterialfv(GL_FRONT, GL_SPECULAR, white);
  glMaterialf(GL_FRONT, GL_SHININESS, 20.0);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_shininess);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_specular);

  glLightfv(GL_LIGHT0, GL_SPECULAR, KGLObject::glcolor(Qt::gray));
	// better enable specular lighting
  glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, lightDir); 

  KGLObject::color(Qt::white);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glShadeModel(GL_SMOOTH);
//  glDisable(GL_DEPTH_TEST);
  glEnable(GL_DEPTH_TEST);

  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glEnable(GL_CULL_FACE); 
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );   

  KGLObject::color(Qt::red);
  axis[0]=new KGLArrow(0.0,0.0,0.0, 1.0,0.0,0.0, 0.05, 0.1);
  axis[3]=new KGLArrow(-1.0,0.0,0.0, 0.0,0.0,0.0, 0.02, 0.05);
  KGLObject::color(Qt::green);
  axis[1]=new KGLArrow(0.0,0.0,0.0, 0.0,1.0,0.0, 0.05, 0.1);
  axis[4]=new KGLArrow(0.0,-1.0,0.0, 0.0,0.0,0.0, 0.02, 0.05);
  KGLObject::color(Qt::blue);
  axis[2]=new KGLArrow(0.0,0.0,0.0, 0.0,0.0,1.0, 0.05, 0.1);
  axis[5]=new KGLArrow(0.0,0.0,-1.0, 0.0,0.0,0.0, 0.02, 0.05);

//  axis[6]=new KGLAxis();
  axis[6]=0L;

  KGLObject::color(Qt::cyan);
  rotationCenter=new KGLSphere(0.0,0.0,0.0, 0.2);
  KGLObject::color(Qt::white);

  m_isInitialized=true;
}

void KGLUniverse::paintObserver(GLfloat *p, GLfloat *d, GLfloat *o)
{
  KGLArrow a1(p[0],p[1],p[2],
      p[0]+d[0],p[1]+d[1],p[2]+d[2],
      0.05, 0.1);
  a1.setColor(Qt::magenta);
  KGLArrow a2(p[0],p[1],p[2],
      p[0]+o[0],p[1]+o[1],p[2]+o[2],
      0.03, 0.06);
  a2.setColor(Qt::yellow);
  a1.paint();
  a2.paint();
}

void KGLUniverse::paintGL()
{
//kdDebug() << "Universe::paint\n";
  if (disableUpdates) return;
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);        
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glEnable(GL_DEPTH_TEST);

  //  kdDebug() << "pos =    " << pos[0] << " " << pos[1] << " " << pos[2]<<"\n";
  //  kdDebug() << "lookTo = " << pos[0]+dir[0] << " " << pos[1]+dir[1]
  //	<< " " << pos[2]+dir[2] << " ( " << phi << " " << psi << ")\n";

  if ( m_staticObserver && m_currentObserver!=m_currentStaticObserver )
  {  
    unsigned int co=m_currentStaticObserver;
    if (m_customObserver)
    {
      float *custom=m_customObserverMethod();
      m_pos[co][0]=custom[0]+m_customRelX*custom[3];
      m_pos[co][1]=custom[1]+m_customRelX*custom[4];
      m_pos[co][2]=custom[2]+m_customRelX*custom[5];
      m_dir[co][0]=custom[3];
      m_dir[co][1]=custom[4];
      m_dir[co][2]=custom[5],
      m_obs[co][0]=custom[6],
      m_obs[co][1]=custom[7];
      m_obs[co][2]=custom[8];
    }
    gluLookAt( m_pos[co][0], m_pos[co][1], m_pos[co][2],
	m_pos[co][0]+m_dir[co][0], m_pos[co][1]+m_dir[co][1],
	m_pos[co][2]+m_dir[co][2], m_obs[co][0], m_obs[co][1], m_obs[co][2] );
  }
  else 
  {
    if (m_customObserver)
    {
      float *custom=m_customObserverMethod();
      pos[0]=custom[0]+m_customRelX*custom[3];
      pos[1]=custom[1]+m_customRelX*custom[4];
      pos[2]=custom[2]+m_customRelX*custom[5];
      dir[0]=custom[3];
      dir[1]=custom[4];
      dir[2]=custom[5],
      obs[0]=custom[6],
      obs[1]=custom[7];
      obs[2]=custom[8];
    }

    gluLookAt( pos[0], pos[1], pos[2],
	pos[0]+dir[0], pos[1]+dir[1], pos[2]+dir[2],
	obs[0], obs[1], obs[2]);
  }

  if ( m_paintElements & c_Axis )
  {
    axis[0]->paint();
    axis[1]->paint();
    axis[2]->paint();
    axis[3]->paint();
    axis[4]->paint();
    axis[5]->paint();
  }

  rotationCenter->paint();

  if (m_paintElements & c_Obs1)
  {
    if (m_currentObserver==0) paintObserver( pos, dir, obs);
    else paintObserver( m_pos[0], m_dir[0], m_obs[0]);
  }
  if (m_paintElements & c_Obs2)
  {
    if (m_currentObserver==1) paintObserver( pos, dir, obs);
    else paintObserver( m_pos[1], m_dir[1], m_obs[1]);
  }
  if (m_paintElements & c_Obs3)
  {
    if (m_currentObserver==2) paintObserver( pos, dir, obs);
    else paintObserver( m_pos[2], m_dir[2], m_obs[2]);
  }

  for (KGLObject *it=objects->first();it!=0L;it=objects->next())
   it->paint();

  if ( m_paintElements & c_TextCoords )
  {
    QPixmap pixmap(width(),18);
    pixmap.fill(Qt::black);
    QPainter p;
    p.begin(&pixmap);
    p.setPen(Qt::white);
    p.setBrush(Qt::white);
    QString s=QString("x: %1, y: %2, z: %3").arg(pos[0]).arg(pos[1]).arg(pos[2]);
    p.drawText(2,16,s);
    p.end();

    QImage Image=pixmap.convertToImage().mirror(false,true);
    glDrawPixels(width(), 18, GL_RGBA, GL_UNSIGNED_BYTE, Image.bits());
  }
  if (axis[6]) axis[6]->paint();

  glPopMatrix();  
}

void KGLUniverse::resizeGL( int w, int h )
{
  glViewport( 0, 0, w, h );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  //  glFrustum( -1.0, 1.0, -1.0, 1.0, frustumNear, 800.0 );
  //  glFrustum( -2.0, 2.0, -2.0, 2.0, frustumNear, 800.0 );
  gluPerspective(viewAngle,0.75,0.4,800.0);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
}
 
void KGLUniverse::lookTo(GLfloat x, GLfloat y, GLfloat z)
{
  GLfloat r=sqrt((pos[0]-x)*(pos[0]-x)+(pos[1]-y)*(pos[1]-y)+(pos[2]-z)*(pos[2]-z));
  phi=(((x-pos[0])==0) ? 0 : atan2((x-pos[0]),(y-pos[1])));

//  GLfloat psi=2*M_PI*v/360;

  psi=(((z-pos[2])==0) ? 0 : atan((z-pos[2])/sqrt((pos[1]-y)*(pos[1]-y)+(pos[0]-x)*(pos[0]-x))));
    
//  printf("::lookTo %g\t%g\t%g\n",pos[0],pos[1],pos[2]);
//  printf("::lookTo %g\t%g\n",phi,psi);

  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);

  obs[0]=-sin(psi)*sin(phi);
  obs[1]=-sin(psi)*cos(phi);
  obs[2]=cos(psi);
  
  distanceToCenter=r;
  
//  printf("pos : %g\t%g\t%g\n",pos[0],pos[1],pos[2]);
//  printf("dir : %g\t%g\t%g\n",dir[0],dir[1],dir[2]);
//  printf("obs : %g\t%g\t%g\n",obs[0],obs[1],obs[2]);
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
}

void KGLUniverse::moveX(int v)
{
  pos[0]=v/1000.0;
  updateGL();   
}

void KGLUniverse::moveY(int v)
{
  pos[1]=v/1000.0;
  updateGL();   
}

void KGLUniverse::moveZ(int v)
{
  pos[2]=v/1000.0;
  updateGL();   
}

void KGLUniverse::movePhi(int v)
{
  phi=v*2*M_PI/360000.0;
  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);
  updateGL();   
}

void KGLUniverse::movePsi(int v)
{
  psi=v*2*M_PI/360000.0;
  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);
  updateGL();   
}

void KGLUniverse::resetTwist()
{
  prevTwist=0;

  emit twistChanged(0);
}

void KGLUniverse::twist(int v)
{
  if (v==0) return;

  GLfloat t=v*2*M_PI/360000.0-prevTwist;
  prevTwist+=t;

  // Now we rotate obs over the axis given by dir, v rad.

  GLfloat q0=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2])/tan(t/2);
  // b=q0*obs + (obs x dir)
  GLfloat b[3];
  b[0]=q0*obs[0]+obs[1]*dir[2]-dir[1]*obs[2];
  b[1]=q0*obs[1]+dir[0]*obs[2]-obs[0]*dir[2];
  b[2]=q0*obs[2]+obs[0]*dir[1]-dir[0]*obs[1];
  // a= -obs.dir
  GLfloat a=-(obs[0]*dir[0]+obs[1]*dir[1]+obs[2]*dir[2]);

  // r= ( q0 * b ) + ( a * -dir) + ( -dir x b )
  obs[0]=q0*b[0]-a*dir[0]+(-dir[1])*b[2]-b[1]*(-dir[2]);
  obs[1]=q0*b[1]-a*dir[1]+b[0]*(-dir[2])-(-dir[0])*b[2];
  obs[2]=q0*b[2]-a*dir[2]+(-dir[0])*b[1]-b[0]*(-dir[1]);

  normalizeVector(obs);

  updateGL();   

}

void KGLUniverse::setViewAngle(int v)
{
  viewAngle=v/1000.0;

  //  printf("view Angle %g\n",viewAngle);
  //  glPushMatrix();
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  //  glFrustum( -1.0, 1.0, -1.0, 1.0, frustumNear, 800.0 );
  //  glOrtho(-1.0, 1.0, -1.0, 1.0, -10.0, 800.0 );
  gluPerspective(viewAngle,0.75,0.4,800.0);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  //  glPopMatrix();

  updateGL(); 
}

void KGLUniverse::setDistanceToCenter(int v)
{
  if (v<=0) return;
  m_delay->stop();
  distanceToCenter=v/1000.0;
  rotationCenter->moveTo(pos[0]+distanceToCenter*dir[0],
      pos[1]+distanceToCenter*dir[1],pos[2]+distanceToCenter*dir[2]);
  rotationCenter->show();

  m_delay->start(1800,true);
  updateGL();
}

void KGLUniverse::setSpeed(int v)
{
  m_speed=v/15.0;
}

void KGLUniverse::hideRotationCenter()
{
  rotationCenter->hide();
  updateGL();
}

void KGLUniverse::moveForward()
{
  if ( m_customObserver && m_customObserverAllowRelativeMovement )
    m_customRelX+=m_speed;

  pos[0]+=m_speed*dir[0];
  pos[1]+=m_speed*dir[1];
  pos[2]+=m_speed*dir[2];

  disableUpdates=true;
  emitCoords();
  disableUpdates=false;

  updateGL();
}

void KGLUniverse::lookToOrigin()
{
  if ( m_customObserver && m_customObserverAllowRelativeMovement )
  {
    m_customRelX=0; 
  }
  lookTo(0.0,0.0,0.0);
}

void KGLUniverse::moveBackward()
{
  if ( m_customObserver && m_customObserverAllowRelativeMovement )
    m_customRelX-=m_speed;

  pos[0]-=m_speed*dir[0];
  pos[1]-=m_speed*dir[1];
  pos[2]-=m_speed*dir[2];
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;

  updateGL();
}

void KGLUniverse::emitCoords()
{
  emit movedX(pos[0]*1000);
  emit movedY(pos[1]*1000);
  emit movedZ(pos[2]*1000);
//  emit phiChanged(phi*360000.0/(2.0*M_PI));
//  emit psiChanged(psi*360000.0/(2.0*M_PI));
  emit distanceToCenterChanged(distanceToCenter*1000);
  
}

void KGLUniverse::moveViewPoint(GLfloat x,GLfloat y,GLfloat z)
{
  pos[0]=x;
  pos[1]=y;
  pos[2]=z;
  updateGL();
  emitCoords();
}

void KGLUniverse::mousePressEvent(QMouseEvent *ev)
{
  if (ev->button() == MidButton)
  {
    mouseRotCenterX=ev->x();
    mouseRotCenterY=ev->y();
    mouseRotPhi=phi;
    mouseRotPsi=psi;
    mouseState=mouseMMBAction;
  }
  else if (ev->button() == LeftButton)
  {
    mouseRotCenterX=ev->x();
    mouseRotCenterY=ev->y();
    mouseRotPhi=phi;
    mouseRotPsi=psi;
    mouseState=mouseLMBAction;
  }
  else if (ev->button() == RightButton)
  {
    QPopupMenu *options=new QPopupMenu(this);
    KSelectAction *modesLMB=new KSelectAction( i18n("Left Mouse Button"),
	0, options);
    KSelectAction *modesMMB=new KSelectAction( i18n("Middle Mouse Button"),
	0, options);
    QStringList strlist;
    strlist.append(i18n("Move X"));
    strlist.append(i18n("Move Y"));
    strlist.append(i18n("Move Z"));
    strlist.append(i18n("Twist"));
    strlist.append(i18n("Rotate Looking Direction"));
    strlist.append(i18n("Rotate Point of View"));
    strlist.append(i18n("Move Distance to Center of Rotation"));
    modesLMB->setItems(strlist);
    modesMMB->setItems(strlist);
    
    connect(modesLMB, SIGNAL(activated(int)), this, SLOT(setLMBAction(int)));
    connect(modesMMB, SIGNAL(activated(int)), this, SLOT(setMMBAction(int)));

    modesLMB->plug(options);
    modesLMB->setCurrentItem(static_cast<int>( mouseLMBAction ));
    modesMMB->plug(options);
    modesMMB->setCurrentItem(static_cast<int>( mouseMMBAction ));

    options->insertSeparator();

    KSelectAction *changeobserver=m_changeObserver;

    if (!m_changeObserver)
      changeobserver=m_changeObserver=new KSelectAction( i18n("Observers") );

    strlist.clear();
    for (int i=0;i<c_NumObservers;i++) strlist << QString::number(i+1);
    changeobserver->setItems(strlist);
    changeobserver->setCurrentItem(m_currentObserver);

    connect(changeobserver, SIGNAL(activated()),
	this, SLOT(changeObserver()));

    changeobserver->plug(options);

    KToggleAction *staticobserver=new KToggleAction( i18n("Static Observer"),
	0, 0, options, "staticObserver");
    connect(staticobserver, SIGNAL(toggled(bool)),
	this, SLOT(setStaticObserver(bool)));
    staticobserver->setEnabled(true);
    staticobserver->plug(options);
    staticobserver->setChecked(m_staticObserver);

    if (!m_customObserverName.isEmpty())
    {
      KToggleAction *observer=new KToggleAction(m_customObserverName, 0, 0,
	  options, "customObserver");
      connect(observer, SIGNAL(toggled(bool)),
	  this, SLOT(setCustomObserver(bool)));
      observer->setEnabled(true);
      observer->plug(options);
      observer->setChecked(m_customObserver);
    }

    KSelectAction *drawoptions=new KSelectAction( i18n("Draw Options"),
	0, options);
    strlist.clear();
    strlist << i18n("Toggle Axis");
    for (int i=0;i<c_NumObservers;i++)
      strlist << i18n("Toggle Observer %1").arg(i+1);
    strlist << i18n("Toggle Coords View");
    drawoptions->setItems(strlist);

    connect(drawoptions, SIGNAL(activated(int)),
	this, SLOT(toggleDrawElements(int)));

    drawoptions->plug(options);
    //  drawoptions->setCurrentItem(m_currentObserver);
    
    options->exec(mapToGlobal(ev->pos()));
//    delete options;
  }
}

void KGLUniverse::setLMBAction(int a)
{
  mouseLMBAction=static_cast<mouseActions>(a);
}

void KGLUniverse::setMMBAction(int a)
{
  mouseMMBAction=static_cast<mouseActions>(a);
}

void KGLUniverse::normalizeVector(GLfloat *v)
{
  GLfloat norm=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);

  v[0]/=norm;
  v[1]/=norm;
  v[2]/=norm;
}

void KGLUniverse::multiplyVector(GLfloat *v, GLfloat m)
{
  v[0]*=m;
  v[1]*=m;
  v[2]*=m;
}

void KGLUniverse::rotatePointOfView(GLfloat x, GLfloat y)
{
  GLfloat p[3];
  GLfloat norm=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);

  p[0]=-dir[0]/norm;
  p[1]=-dir[1]/norm;
  p[2]=-dir[2]/norm;

  // p is now the point in the unit Sphere which we want to rotate.
  // obs defines an orientation which is "up" on ev->y() (obs==e2)

  GLfloat e2[3];
  e2[0]=obs[0];
  e2[1]=obs[1];
  e2[2]=obs[2];

  // e1 is  e2 x p , that is, the "right" direction on ev->x()
  GLfloat e1[3];
  e1[0]=e2[1]*p[2]-p[1]*e2[2];
  e1[1]=p[0]*e2[2]-e2[0]*p[2];
  e1[2]=e2[0]*p[1]-p[0]*e2[1];

//  printf("e1 (%g %g %g) \n",e1[0],e1[1],e1[2]);
//  printf("e2 (%g %g %g) \n",e2[0],e2[1],e2[2]);
  normalizeVector(e1);
  normalizeVector(e2);

//  printf("e1 %g %g %g \n",e1[0],e1[1],e1[2]);
//  printf("e2 %g %g %g \n",e2[0],e2[1],e2[2]);

//  printf("x %g  \n",x);
//  printf("y %g  \n",y);

  multiplyVector(e1,x);
  multiplyVector(e2,y);

//  printf("e1,,%g %g %g \n",e1[0],e1[1],e1[2]);
//  printf("e2,,%g %g %g \n",e2[0],e2[1],e2[2]);

  GLfloat r[3];
  r[0]=e1[0]+e2[0];
  r[1]=e1[1]+e2[1];
  r[2]=e1[2]+e2[2];
//  printf("r %g %g %g \n",r[0],r[1],r[2]);

  // v= p x r
  GLfloat v[3];
  v[0]=p[1]*r[2]-r[1]*p[2];
  v[1]=r[0]*p[2]-p[0]*r[2];
  v[2]=p[0]*r[1]-r[0]*p[1];

//  printf("v %g %g %g \n",v[0],v[1],v[2]);

  // v is now the axis around which we want to rotate a distance given by 
  // the (Manhattan ?) norm of ( ev->x() ev->y() )

  GLfloat xrad=fabs(x*M_PI/width());
  GLfloat yrad=fabs(y*M_PI/height());
  GLfloat alpha= QMAX(xrad, yrad);

//  printf("alpha %g (xrad %g , yrad %g)\n",alpha,xrad,yrad);
  // Now we rotate p over the axis given by v, alpha rad.

  GLfloat q0=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])/tan(alpha/2);
  // b=q0*p + (p x v) 
  GLfloat b[3];
  b[0]=q0*p[0]+p[1]*v[2]-v[1]*p[2];
  b[1]=q0*p[1]+v[0]*p[2]-p[0]*v[2];
  b[2]=q0*p[2]+p[0]*v[1]-v[0]*p[1];
  // a= -p.v
  GLfloat a=-(p[0]*v[0]+p[1]*v[1]+p[2]*v[2]);

  // r= ( q0 * b ) + ( a * -v) + ( -v x b )
  r[0]=q0*b[0]-a*v[0]+(-v[1])*b[2]-b[1]*(-v[2]);
  r[1]=q0*b[1]-a*v[1]+b[0]*(-v[2])-(-v[0])*b[2];
  r[2]=q0*b[2]-a*v[2]+(-v[0])*b[1]-b[0]*(-v[1]);

  normalizeVector(r);

//  printf("r %g %g %g\n",r[0],r[1],r[2]);

  // We have in r the rotated p point (over the unit sphere).

  pos[0]=pos[0]+distanceToCenter*(dir[0]+r[0]);
  pos[1]=pos[1]+distanceToCenter*(dir[1]+r[1]);
  pos[2]=pos[2]+distanceToCenter*(dir[2]+r[2]);

  dir[0]=-r[0];
  dir[1]=-r[1];
  dir[2]=-r[2];

  // We are almost finished, we only have to update obs to be
  // orthogonal to dir

  // We're doing (dir x obs) x dir to get a vector v which will
  // be in the "direction" of obs (in fact, it's not exactly on the
  // same direction, but in the plane generated by obs and dir, and
  // orthogonal to dir)

  b[0]=dir[1]*obs[2]-obs[1]*dir[2];
  b[1]=obs[0]*dir[2]-dir[0]*obs[2];
  b[2]=dir[0]*obs[1]-obs[0]*dir[1];

  v[0]=b[1]*dir[2]-dir[1]*b[2];
  v[1]=dir[0]*b[2]-b[0]*dir[2];
  v[2]=b[0]*dir[1]-dir[0]*b[1];

  obs[0]=v[0];
  obs[1]=v[1];
  obs[2]=v[2];

  normalizeVector(obs);

  phi=( ( dir[0]==0 ) ? 0 : atan2(dir[0],dir[1]) );
  psi=( ( dir[2]==0 ) ? 0 : atan( dir[2]/sqrt(dir[1]*dir[1]+dir[0]*dir[0])) );

  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
  updateGL();
}

void KGLUniverse::rotateLookDirection(GLfloat x,GLfloat y)
{
  GLfloat rad=viewAngle*M_PI/360.0;
  // GLfloat dphi=(mouseRotCenterX-ev->x())*M_PI/((viewAngle/4.0)*width()/2.0);
  // GLfloat dpsi=(ev->y()-mouseRotCenterY)*M_PI/((viewAngle/4.0)*height()/2.0);

  //   GLfloat dphi=(mouseRotCenterX-ev->x())*rad/(width()/2.0);
  //   GLfloat dpsi=(ev->y()-mouseRotCenterY)*rad/(height()/2.0);
  //   phi=mouseRotPhi+dphi;
  //   psi=mouseRotPsi+dpsi;

  //   dir[0]=cos(psi)*sin(phi);
  //   dir[1]=cos(psi)*cos(phi);
  //   dir[2]=sin(psi);
   
  GLfloat p[3];
  GLfloat norm=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);

  p[0]=dir[0]/norm;
  p[1]=dir[1]/norm;
  p[2]=dir[2]/norm;

  // p is now the point in the unit Sphere which we are looking to

  // obs defines an orientation which is "up" on ev->y() (obs==e2)

  GLfloat e2[3];
  e2[0]=obs[0];
  e2[1]=obs[1];
  e2[2]=obs[2];

  // e1 is p x e2 , that is, the "right" direction on ev->x()
  GLfloat e1[3];
  e1[0]=-e2[1]*p[2]+p[1]*e2[2];
  e1[1]=-p[0]*e2[2]+e2[0]*p[2];
  e1[2]=-e2[0]*p[1]+p[0]*e2[1];

//  printf("e1 (%g %g %g) \n",e1[0],e1[1],e1[2]);
//  printf("e2 (%g %g %g) \n",e2[0],e2[1],e2[2]);
  normalizeVector(e1);
  normalizeVector(e2);

//  printf("e1 %g %g %g \n",e1[0],e1[1],e1[2]);
//  printf("e2 %g %g %g \n",e2[0],e2[1],e2[2]);

//  printf("x %g  \n",x);
//  printf("y %g  \n",y);

  multiplyVector(e1,x);
  multiplyVector(e2,y);

//  printf("e1,,%g %g %g \n",e1[0],e1[1],e1[2]);
//  printf("e2,,%g %g %g \n",e2[0],e2[1],e2[2]);

  GLfloat r[3];
  r[0]=e1[0]+e2[0];
  r[1]=e1[1]+e2[1];
  r[2]=e1[2]+e2[2];
//  printf("r %g %g %g \n",r[0],r[1],r[2]);

  // v= p x r
  GLfloat v[3];
  v[0]=p[1]*r[2]-r[1]*p[2];
  v[1]=r[0]*p[2]-p[0]*r[2];
  v[2]=p[0]*r[1]-r[0]*p[1];

//  printf("v %g %g %g \n",v[0],v[1],v[2]);

  // v is now the axis around which we want to rotate a distance given by 
  // the (Manhattan ?) norm of ( ev->x() ev->y() )

  GLfloat xrad=fabs(x*rad/(width()/2.0));
  GLfloat yrad=fabs(y*rad/(height()/2.0));
  GLfloat alpha= QMAX(xrad, yrad);

//  printf("alpha %g (xrad %g , yrad %g)\n",alpha,xrad,yrad);
  // Now we rotate p over the axis given by v, alpha rad.

  GLfloat q0=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])/tan(alpha/2);
  // b=q0*p + (p x v) 
  GLfloat b[3];
  b[0]=q0*p[0]+p[1]*v[2]-v[1]*p[2];
  b[1]=q0*p[1]+v[0]*p[2]-p[0]*v[2];
  b[2]=q0*p[2]+p[0]*v[1]-v[0]*p[1];
  // a= -p.v
  GLfloat a=-(p[0]*v[0]+p[1]*v[1]+p[2]*v[2]);

  // r= ( q0 * b ) + ( a * -v) + ( -v x b )
  r[0]=q0*b[0]-a*v[0]+(-v[1])*b[2]-b[1]*(-v[2]);
  r[1]=q0*b[1]-a*v[1]+b[0]*(-v[2])-(-v[0])*b[2];
  r[2]=q0*b[2]-a*v[2]+(-v[0])*b[1]-b[0]*(-v[1]);

  normalizeVector(r);

//  printf("r %g %g %g\n",r[0],r[1],r[2]);

  // We have in r the rotated p point (over the unit sphere).

  dir[0]=r[0];
  dir[1]=r[1];
  dir[2]=r[2];

  // We are almost finished, we only have to update obs to be
  // orthogonal to dir

  // We're doing (dir x obs) x dir to get a vector v which will
  // be in the direction of obs.

  b[0]=dir[1]*obs[2]-obs[1]*dir[2];
  b[1]=obs[0]*dir[2]-dir[0]*obs[2];
  b[2]=dir[0]*obs[1]-obs[0]*dir[1];

  v[0]=b[1]*dir[2]-dir[1]*b[2];
  v[1]=dir[0]*b[2]-b[0]*dir[2];
  v[2]=b[0]*dir[1]-dir[0]*b[1];

  normalizeVector(v);

  obs[0]=v[0];
  obs[1]=v[1];
  obs[2]=v[2];

  phi=( ( dir[0]==0 ) ? 0 : atan2(dir[0],dir[1]) );
  psi=( ( dir[2]==0 ) ? 0 : atan( dir[2]/sqrt(dir[1]*dir[1]+dir[0]*dir[0])) );

  updateGL();

  emit phiChanged((phi)*360000.0/(2.0*M_PI));
  emit psiChanged((psi)*360000.0/(2.0*M_PI)); 
}

void KGLUniverse::mouseMoveEvent(QMouseEvent *ev)
{
  switch (mouseState)
  {
    case (ac_rotatePointOfView) :
      {
	rotatePointOfView( ev->x()-mouseRotCenterX, ev->y()-mouseRotCenterY );
	mouseRotCenterX=ev->x();
	mouseRotCenterY=ev->y();
      } break;
    case (ac_rotateLookDirection) : 
      {
	rotateLookDirection( ev->x()-mouseRotCenterX, mouseRotCenterY-ev->y() );
	mouseRotCenterX=ev->x();
	mouseRotCenterY=ev->y();
      } break;
    default: kdDebug() << i18n("Not Implemented yet") << endl;
  }
}

void KGLUniverse::mouseReleaseEvent(QMouseEvent *)
{
}

void KGLUniverse::changeObserver()
{
  int i=m_changeObserver->currentItem();
  if (m_currentObserver==(unsigned int) i ) return;

  // Store the state of this observer

  m_pos[m_currentObserver][0]=pos[0];
  m_pos[m_currentObserver][1]=pos[1];
  m_pos[m_currentObserver][2]=pos[2];
  m_viewAngle[m_currentObserver]=viewAngle;
  m_distanceToCenter[m_currentObserver]=distanceToCenter;
  m_dir[m_currentObserver][0]=dir[0];
  m_dir[m_currentObserver][1]=dir[1];
  m_dir[m_currentObserver][2]=dir[2];
  m_obs[m_currentObserver][0]=obs[0];
  m_obs[m_currentObserver][1]=obs[1];
  m_obs[m_currentObserver][2]=obs[2];
  
  m_currentObserver=i;

  pos[0]=m_pos[m_currentObserver][0];
  pos[1]=m_pos[m_currentObserver][1];
  pos[2]=m_pos[m_currentObserver][2];
  viewAngle=m_viewAngle[m_currentObserver];
  distanceToCenter=m_distanceToCenter[m_currentObserver];
  dir[0]=m_dir[m_currentObserver][0];
  dir[1]=m_dir[m_currentObserver][1];
  dir[2]=m_dir[m_currentObserver][2];
  obs[0]=m_obs[m_currentObserver][0];
  obs[1]=m_obs[m_currentObserver][1];
  obs[2]=m_obs[m_currentObserver][2];
  
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
  updateGL(); 

}

void KGLUniverse::setStaticObserver(bool b)
{
  m_staticObserver=b;
  m_currentStaticObserver=m_currentObserver;
}

void KGLUniverse::setCustomObserver(bool b)
{
//  pos[0]=0; pos[1]=0; pos[2]=0;
//  dir[0]=0; dir[1]=0; dir[2]=0;
//  obs[0]=0; obs[1]=0; obs[2]=0;
  m_customObserver=b;
}

void KGLUniverse::toggleDrawElements(int i)
{
  int t=1<<i;
  m_paintElements=m_paintElements ^ t;
}

void KGLUniverse::enableDrawElements(int i)
{
  int t=1<<i;
  m_paintElements|=t;
}

void KGLUniverse::disableDrawElements(int i)
{
  int t=1<<i;
  m_paintElements&=~t;
}

void KGLUniverse::insertObject(KGLObject *object)
{
  objects->append(object);
  object->plug(this,objects->at());
}

void KGLUniverse::removeObject(int id)
{
  objects->remove(id);
  
  for (KGLObject *it=objects->first();it!=0L;it=objects->next())
    it->setID(objects->at());
}

void KGLUniverse::setCustomObserver(const QString &name,
	float * (*observerCoords)())
{
  m_customObserverMethod=observerCoords;
  m_customObserverName=name;
}
