/*  kgluniverse.h - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#ifndef _KGLUNIVERSE_H_
#define _KGLUNIVERSE_H_

#include <qgl.h>
#include <qlist.h>
#include <qsize.h>
#include "kglobject.h"

class QImage;
class KAction;

typedef float * (*ObserverCoordsCallback)();
class KGLUniverse : public QGLWidget
{
  Q_OBJECT  
public:

    KGLUniverse( QWidget* parent, const char* name );
    virtual ~KGLUniverse();

    void	lookTo(GLfloat x,GLfloat y, GLfloat z);
    void	moveViewPoint(GLfloat x, GLfloat y, GLfloat z);
    virtual QSize sizeHint() const { return QSize(300,300); };

    void 	rotatePointOfView(GLfloat x, GLfloat y);
    void 	rotateLookDirection(GLfloat x, GLfloat y);

    void 	insertObject(KGLObject *o);
    void        removeObject(int id);
    
    void 	setCustomObserver(const QString &name,
	float * (*observerCoords)());

    void	emitCoords();

    bool 	customObserver() const { return m_customObserver; };
    bool 	staticObserver() const { return m_staticObserver; };

    enum { c_Axis = 1, c_Obs1=2, c_Obs2=4, c_Obs3=8, c_TextCoords=16 };
    enum mouseActions { ac_moveX=0, ac_moveY, ac_moveZ, ac_Twist, 
      ac_rotateLookDirection, ac_rotatePointOfView, ac_moveDistanceToCenter};
protected:

    virtual void initializeGL();
    virtual void paintGL();
    virtual void paintObserver(GLfloat *p, GLfloat *d, GLfloat *o);
    virtual void resizeGL( int w, int h );
 
    virtual void  mousePressEvent( QMouseEvent *ev);
    virtual void  mouseMoveEvent( QMouseEvent *ev);
    virtual void  mouseReleaseEvent( QMouseEvent *ev);

    QList <KGLObject> *objects;
    bool        m_isInitialized;
public slots:
    void	moveX(int x);
    void	moveY(int y);
    void	moveZ(int z);
    void	movePhi(int v);
    void	movePsi(int v);
    void	twist(int v);
    void	resetTwist();
    void	setViewAngle(int v);
    void	setSpeed(int v);
    void	setDistanceToCenter(int v);
    void	moveForward();
    void	moveBackward();
    void	lookToOrigin();
    void	hideRotationCenter();
    void 	setLMBAction(int a);
    void 	setMMBAction(int a);
    void	changeObserver();
    void 	setStaticObserver(bool b);
    void 	setCustomObserver(bool b);
    void 	toggleDrawElements(int);
    void 	enableDrawElements(int);
    void 	disableDrawElements(int);
        
signals:
    /**
     * The following signals are emitted when each parameter is changed
     * _internally_ (for example, by @ref lookTo )
     */
    void 	movedX(int v);
    void 	movedY(int v);
    void 	movedZ(int v);
    void	phiChanged(int v);
    void	psiChanged(int v);
    void	twistChanged(int v);
    void	distanceToCenterChanged(int v);

private:

    void normalizeVector(GLfloat *v);
    void multiplyVector(GLfloat *v,GLfloat m);

    GLfloat pos[3];
    GLfloat dir[3];
    GLfloat obs[3];
    GLfloat phi,psi;
    GLfloat mouseRotCenterX,mouseRotCenterY;
    GLfloat mouseRotPhi,mouseRotPsi;
    GLfloat prevTwist;
    GLfloat viewAngle;
    GLfloat distanceToCenter;

    enum { c_NumObservers = 3 };
    GLfloat m_pos[c_NumObservers][3];
    GLfloat m_dir[c_NumObservers][3];
    GLfloat m_obs[c_NumObservers][3];
    GLfloat m_viewAngle[c_NumObservers];
    GLfloat m_distanceToCenter[c_NumObservers];
    ObserverCoordsCallback m_customObserverMethod;
    QString m_customObserverName;
    bool m_customObserverAllowRelativeMovement;
    GLfloat m_customRelX;

    unsigned int m_currentObserver;
    unsigned int m_currentStaticObserver;
    bool m_staticObserver;
    bool m_customObserver;
    
    unsigned int m_paintElements;

    KGLObject *axis[7];
    KGLObject *rotationCenter;
    GLfloat phi_rot;
    bool disableUpdates;
    GLfloat m_speed;
    

    mouseActions mouseState;
    mouseActions mouseLMBAction;
    mouseActions mouseMMBAction;


    class QTimer *m_delay;

    class KSelectAction *m_changeObserver;
}; 

#endif
