/*  mapbrowser.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "mapbrowser.h"
#include <kdebug.h>
#include <klocale.h>
#include <qtimer.h>
#include <qwidget.h>
#include "map.h"
#include "parameter.h"

MapBrowser::MapBrowser(QWidget *parent) : QObject(parent,"helpBrowser")
{ 
  html=new KHTMLPart(parent);
  connect( html->browserExtension(),
      SIGNAL( openURLRequest( const KURL &, const KParts::URLArgs & ) ),
      this, SLOT( helpOpenURL( const KURL & ) ) ); 
      
}

void MapBrowser::helpOpenURL(const KURL &url)
{
  kdDebug() << url.path() << endl;
  kdDebug() << url.host() << endl;
  if (url.protocol()=="map")
  {
    kdDebug() << url.path() << endl;
    kdDebug() << url.host() << endl;

    html->begin(KURL("file:/home/mn4mat/opt/kde2/share/apps/kalamaris/"));
    html->write(Map::maps.find(url.path())->help());
    html->end();
    
/*    QString name("Cos");
    QString p(Map::helpPrefix());
    p+=QString("<FORM METHOD=\"GET\" ACTION=\"math://PlotFunction/%1\">").arg(name);
    p+=QString("<H3>Plot Function <A HREF=\"map:Cos\">%1</A> Request</H3>").arg(name);
    p+=QString("<P>From    <INPUT TYPE=TEXT NAME=a>\n"
	"<P>To      <INPUT TYPE=TEXT NAME=b>\n"
	"<P>Samples <INPUT TYPE=TEXT NAME=m>\n"
	"<P><DIV ALIGN=RIGHT><INPUT TYPE=\"SUBMIT\" VALUE=\"Plot\">\n"
	"<P>\n"
	"<INPUT TYPE=RESET VALUE=\"Reset\">"
	"</DIV></FORM>\n");
    p+=Map::helpPostfix();

    kdDebug() << p << endl;

    //  emit writeHelp(p);

    begin();
    write(p);
    end();

*/
  }
  else if (url.protocol()=="math")
  {
    if (url.host()=="RequestPlotFunction") // Something like math://RequestPlotFunction/Cos
    {
//      requestPlotFunction(url.path().mid(1));
      m_fname=url.path().mid(1);
      QTimer::singleShot(0,this,SLOT(requestPlotFunction()));
    }
    else
    if (url.host()=="PlotFunction") // Something like math://PlotFunction/Cos
    {
      kdDebug() << url.path() << endl;
      kdDebug() << url.query() << endl;
      plotFunction(url.path().mid(1),url.query());
     
    }
  }
  else if (url.protocol()=="demo")
  {
    if (url.host()=="Pendulum") // Something like demo://Pendulum
    {
      m_fname=url.path().mid(1);
      QTimer::singleShot(0,this,SLOT(demoPendulum()));
    }
    else
    if (url.host()=="Exponential") // Something like math://Exponential
    {
      m_fname=url.path().mid(1);
      QTimer::singleShot(0,this,SLOT(demoExponential()));
    }
    else
    {
      QTimer::singleShot(0,this,SLOT(showDemoIndex()));
    }
  }
  
}

void MapBrowser::requestPlotFunction()
{
  QString name(m_fname);
  QString p(Map::helpPrefix());
  p+=QString("<FORM METHOD=\"GET\" ACTION=\"math://PlotFunction/%1\">").arg(name);
  p+=QString(i18n("<H3>Plot Function %1 Request</H3>")).arg(name);
  QString x;
  Map *map=Map::maps.find(name);
  if ( map && map->parameter(0) ) x=map->parameter(0)->name();
      else x="x";
  p+=QString("<P>Variable <INPUT TYPE=TEXT NAME=x VALUE=\"%1\">\n").arg(x);
  p+=QString(i18n("<P>From     <INPUT TYPE=TEXT NAME=a VALUE=\"-5\">\n"
     "<P>To       <INPUT TYPE=TEXT NAME=b VALUE=\"5\">\n"
     "<P>Samples  <INPUT TYPE=TEXT NAME=m VALUE=\"30\">\n"
     "<H3>Type of Plot :</H3>"
     "<BR><INPUT TYPE=RADIO NAME=plottype VALUE=\"plot\" CHECKED>Plot"
     "<BR><INPUT TYPE=RADIO NAME=plottype VALUE=\"animation\">Animate"
     "<P><DIV ALIGN=RIGHT><INPUT TYPE=\"SUBMIT\" VALUE=\"Plot\">\n"
     "<P>\n"
     "<INPUT TYPE=RESET VALUE=\"Reset\">"
     "</DIV></FORM>\n"));
  p+=Map::helpPostfix();
  
  html->begin(KURL("file:/home/mn4mat/opt/kde2/share/apps/kalamaris/"));
  html->write(p);
  html->end();
 
}

QString MapBrowser::getKey(const QStringList &parameters, const QString &key) const
{
  return parameters.grep(QRegExp(QString("^%1=.*$").arg(key)))[0].mid(key.length()+1);
}

QString MapBrowser::getNumKey(const QStringList &parameters, const QString &key) const
{
  return parameters.grep(QRegExp(QString("^%1=-\?[0-9]+\\.\?[0-9]*$").arg(key)))[0]
    		.mid(key.length()+1) ;
}
void MapBrowser::plotFunction(const QString &name, const QString &query)
{
  QStringList parameters=QStringList::split('&',query);

  QString x( getKey(parameters,"x") );
  QString a( getNumKey(parameters,"a") );
  QString b( getNumKey(parameters,"b") );
  QString m( getNumKey(parameters,"m") );
  QString plottype( getKey(parameters,"plottype") );
/*  QString a( parameters.grep(QRegExp("^a=-\?[0-9]+\\.\?[0-9]*$"))[0].mid(2) );
  QString b( parameters.grep(QRegExp("^b=-\?[0-9]+\\.\?[0-9]*$"))[0].mid(2) );
  QString m( parameters.grep(QRegExp("^m=-\?[0-9]+\\.\?[0-9]*$"))[0].mid(2) );
  */
  kdDebug() << QString("a=(%1)   b=(%2)   m=(%3)").arg(a).arg(b).arg(m) << endl;
  if ((a.isEmpty())||(b.isEmpty())||(m.isEmpty())) return; // XXX Should show an error box

  emit process(QString("_m=EvalFunc(%1,%2,%3,%4,%5)")
      	.arg(Map::maps.find(name)->string()).arg(x).arg(a).arg(b).arg(m));
  if (plottype=="Plot")
    emit process(QString("PlotData(_m[0],_m[1])"));
  else
    emit process(QString("AnimateData(_m[0],_m[1])"));
  emit process(QString("Clear(_m)"));
}

void MapBrowser::demoPendulum()
{
}

void MapBrowser::demoExponential()
{
}

void MapBrowser::showDemoIndex()
{
  QString p(Map::helpPrefix());
  p+=i18n("<CENTER><H2>Examples</H2></CENTER><P><H3><I>"
      "<P><A HREF=\"demo://Exponential\">Exponential</A>"
      "<P><A HREF=\"demo://Circunference\">Circunference</A>"
      "<P><A HREF=\"demo://Pendulum\">Pendulum</A>"
      "</I></H3>");
  p+=Map::helpPostfix();
  
 
  html->begin(KURL("file:/home/mn4mat/opt/kde2/share/apps/kalamaris/"));
  html->write(p);
  html->end();
}
