/*  mapvararg.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "mapvararg.h"
#include "parameter.h"
#include "variable.h"
#include <kdebug.h>

MapVariableArguments::MapVariableArguments ( const QString &decl, 
		    const QString &nam, bool parsedefinition ) 
			: Map( decl, QStringList(), nam, parsedefinition)
{
  m_maxParameters=3;
  delete [] m_parameters;
  m_parameters=new Parameter *[m_maxParameters];
  for (unsigned int i=0;i<m_maxParameters;i++) m_parameters[i]=0L;
}

MapVariableArguments::MapVariableArguments (const MapVariableArguments &m) : Map(m)
{
  m_maxParameters=m_nParameters;
  if (m_maxParameters==0)
  {
    m_maxParameters=3;
    delete [] m_parameters;
    m_parameters=new Parameter *[m_maxParameters];
    for (unsigned int i=0;i<m_maxParameters;i++) m_parameters[i]=0L;
  }
}

MapVariableArguments::~MapVariableArguments()
{
}

Map *MapVariableArguments::copy(void) const
{
  return new MapVariableArguments(*this);
}

void MapVariableArguments::allocParameter(unsigned int i, Parameter *parameter)
{
  if (i<m_nParameters) delete m_parameters[i];
  else if (i>=m_maxParameters)
  {
    // XXX Could this code cause a race-condition when Kalamaris evolve ? 
    Parameter **paramsback=m_parameters;
    while (m_maxParameters<=i) m_maxParameters*=2;
    m_parameters=new Parameter *[m_maxParameters];
    unsigned int j;
    for ( j=0;j<m_nParameters;j++) m_parameters[j]=paramsback[j];
    for (   ;j<m_maxParameters;j++) m_parameters[j]=0L;
    delete [] paramsback;
  }
  m_parameters[i]=parameter;
  if (i+1>m_nParameters) m_nParameters=i+1;
}

void MapVariableArguments::freeParameter(unsigned int i)
{
  delete m_parameters[i];
  for ( unsigned int j=i+1 ; j<m_nParameters; j++) m_parameters[j-1]=m_parameters[j];
  m_parameters[m_nParameters-1]=0L;
  m_nParameters--;
}

void MapVariableArguments::setParameterExpression(unsigned int i,const QString &expr)
{
  QString pname(QString("$%1").arg(i));
  allocParameter(i, new Parameter(pname,expr));
}

void MapVariableArguments::setParameterExpression(unsigned int i, Map *expr)
{
  QString pname(QString("$%1").arg(i));
  allocParameter(i, new Parameter(pname,expr));
}

void MapVariableArguments::setParameterExpression(unsigned int i, Parameter *expr)
{
  allocParameter(i, expr);
}

void MapVariableArguments::addParameterName(const QString &name)
{
  allocParameter(m_nParameters, new Parameter(name));
}

void MapVariableArguments::delParameterName(const QString &name)
{
  int i=argumentIndex(name);
  if (i!=-1)  freeParameter(i);
}

int MapVariableArguments::argumentIndex(const QString &name) const
{
  for (unsigned int i=0;i<m_nParameters;i++) 
    if (name==m_parameters[i]->name()) return (int)i;

  return -1; // With this simple line, we are losing the work of using 
	//unsigned ints everywhere, as this function returns a signed int
}

int MapVariableArguments::isArgumentName(const QString &name) const
{
  return (argumentIndex(name)!=-1);
}

void MapVariableArguments::mimetizeFunction(Map *m)
{
//  kdDebug() << QString("MVA::mimetize (%1) (%2)").arg(string()).arg(m->string()) << endl;
  delete m_tree;
  m_tree=m;

  if (!m_tree) return; //XXX
  
//  tree->setForceEvaluation(forceevaluation);
  m_tree->setForceEvaluation(true);
  for (unsigned int i=0;i<m_tree->nArgs();i++) 
    // XXX this and tree must have the same nparams. We should check for it
  {
//    params[i]->setName(m.params[i]->name());
//    kdDebug() << QString("MVA::mimetize.%1.%2.%3.").arg(i).arg(params[i]->name()).arg(params[i]->map()->string()) << endl;
//    tree->setParameterExpression(i,params[i]->map()->copy());
//    tree->setParameterExpression(i,new Variable(QString("$%1").arg(i)));
    m_parameters[i]->setName(m_tree->parameter(i)->name()); /// XXX This line has been replaced by the next one
    // to try to eliminate the side effects of the optimization in Parameter::Parameter
//    *params[i]=*tree->params[i];
//    tree->params[i]=0L;
  }
 
}
 
Map *MapVariableArguments::eval(const QDict <Map> &vars)
{
  Map *r;
//  kdDebug() << QString("MVA::eval ") + name << endl;
//  debugMaps(vars);
  if (m_tree==0L)
  {
    MapVariableArguments *nosubs =
      static_cast<MapVariableArguments *>(vars.find("$$__noSubstitute"));
    QDict <Map> vars2(vars);
    bool deletenosubs=false;
    if (!nosubs)
    {
      nosubs=new MapVariableArguments();
      nosubs->setName("$$__noSubstitute",false);
      vars2.insert(QString("$$__noSubstitute"),nosubs);
      deletenosubs=true;
    }
    else
    {
      if (nosubs->isArgumentName(m_name))
      {
	kdError() << QString("Recursive lazy evaluation (%1)!!").arg(m_name) << endl;
	return copy();
      }
    }
    nosubs->addParameterName(m_name);
    
    Map *m=functionFactory(m_name);
//    m->setForceEvaluation(true);

    mimetizeFunction(m);
    
    r=Map::eval(vars2);

    nosubs->delParameterName(m_name);
    if (deletenosubs) delete nosubs;

    delete m_tree;
    m_tree=0L;
  }
  else
    r=Map::eval(vars);

  return r;
}

QString MapVariableArguments::string(void) const
{
  if (!m_tree)
  {
    const Map *m=maps.find(m_name);
    if (!m) return QString("MVA:null");
    if (m==static_cast<const Map *>(this))
    {
      kdError() << "MVA::string : Recursive Call" << endl;
      return m_name;
    }
    return m->string();
  }
  else
    return Map::string();
}

Map *MapVariableArguments::derive(const Variable &var, const QDict <Map> &vars)
{
  kdDebug() << "MVA::derive" << endl;
  Map *r=0L;
  if (!m_tree)
  {
    Map *m=functionFactory(m_name);
    //    m->setForceEvaluation(true);
    kdDebug() << "MVA::derive2" << endl;
    // XXX We should check for recursive evaluation here as well as we did in ::eval

    mimetizeFunction(m);
    kdDebug() << "MVA::derive3" << endl;
    r=Map::derive(var, vars);

    delete m_tree;
    m_tree=0L;
  }
  else r=Map::derive(var, vars);

  return r;
}

