/*  matrix.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "matrix.h"
#include <klocale.h>
#include "constant.h"
#include <kdebug.h>

Matrix::Matrix(const QString &_m) : Map("","","",FALSE)
{
  m_isAClassName="Matrix";
//  kdDebug() << QString("Matrix constructor %1").arg(_m) << endl;
  QStringList rows=split(';',_m);

  unsigned int rdim=rows.count();
  QStringList row;
  unsigned int i;
  kdDebug() << "Matrix::Matrix" << endl;
  unsigned int cdim=1;

  Map ***maps=new Map **[rdim];
  for (i=0;i<rdim;i++)
  {
    row=split(',',rows[i]);
    cdim=row.count();
    maps[i]=new Map *[cdim];
    for (unsigned int j=0;j<cdim;j++)
    {
      maps[i][j]=Map::parse(row[j]);
//      kdDebug() << QString("%1 %2 (%3)").arg(i).arg(j).arg(maps[i][j]->isA()) << endl;
    }
  }	    
  m_matrix=new TMatrix(rdim,cdim,maps);

//  kdDebug() << QString("Matrix constructor %1(end)").arg(_m) << endl;
//  kdDebug() << m->string() << endl;

}

Matrix::Matrix(TMatrix *matrix) : Map()
{
  m_isAClassName="Matrix";
  m_matrix=matrix;
}

Matrix::Matrix(const TMatrix &m) : Map()
{
  m_isAClassName="Matrix";
  m_matrix=new TMatrix(m);
}

Matrix::Matrix(const Matrix &m) : Map()
{
  m_isAClassName="Matrix";
  m_matrix=m.m_matrix->copy();
}

Map *Matrix::copy(void) const
{
  return new Matrix(*this);
}

Map *Matrix::eval (const QDict <Map> &vars)
{
  TMatrix *mat=new TMatrix(m_matrix->nRows(),m_matrix->nCols());

  for (unsigned int i=0;i<m_matrix->nRows();i++)
    for (unsigned int j=0;j<m_matrix->nCols();j++)
      mat->set(i,j,m_matrix->at(i,j)->eval(vars));
      
  return new Matrix(mat);
}

bool Matrix::isConstant(void) const
{
  for (unsigned int i=0;i<m_matrix->nRows();i++)
    for (unsigned int j=0;j<m_matrix->nCols();j++)
      if (m_matrix->at(i,j)->isConstant()) return false;

  return true;
}

/*Matrix *operator +(const Matrix &m1, const Matrix &m2)
{
  return new Matrix( *m1.m + *m2.m );
}*/

Matrix *Matrix::add(Matrix *m1, Matrix *m2, bool managem1, bool managem2)
{
  TMatrix *mm1,*mm2;
  mm1=m1->m_matrix;
  mm2=m2->m_matrix;
  if (managem1)
  {
    m1->m_matrix=0L;
    delete m1;
  }
  if (managem2)
  {
    m2->m_matrix=0L;
    delete m2;
  }

  return new Matrix( TMatrix::add(mm1,mm2,managem1,managem2) );
}

Matrix *Matrix::sub(Matrix *m1, Matrix *m2, bool managem1, bool managem2)
{
  TMatrix *mm1,*mm2;
  mm1=m1->m_matrix;
  mm2=m2->m_matrix;
  if (managem1)
  {
    m1->m_matrix=0L;
    delete m1;
  }
  if (managem2)
  {
    m2->m_matrix=0L;
    delete m2;
  }

  return new Matrix( TMatrix::sub(mm1,mm2,managem1,managem2) );
}


Matrix *Matrix::mul(Matrix *m1, Matrix *m2, bool managem1, bool managem2)
{
  TMatrix *mm1,*mm2;
  mm1=m1->m_matrix;
  mm2=m2->m_matrix;
  if (managem1)
  {
    m1->m_matrix=0L;
    delete m1;
  }
  if (managem2)
  {
    m2->m_matrix=0L;
    delete m2;
  }

  return new Matrix( TMatrix::mul(mm1,mm2,managem1,managem2) );
}

Matrix *Matrix::mul(Constant *c1, Matrix *m2, bool managem1, bool managem2)
{
  T *cc1;
  TMatrix *mm2;
  cc1=c1->t();
  mm2=m2->m_matrix;
  if (managem1)
  {
    c1->m_c=0L;
    delete c1;
  }
  if (managem2)
  {
    m2->m_matrix=0L;
    delete m2;
  }

  return new Matrix( TMatrix::mul(cc1,mm2,managem1,managem2) );
}

Matrix *Matrix::div(Matrix *m1, Matrix *m2, bool managem1, bool managem2)
{
  TMatrix *mm1,*mm2;
  mm1=m1->m_matrix;
  mm2=m2->m_matrix;
  if (managem1)
  {
    m1->m_matrix=0L;
    delete m1;
  }
  if (managem2)
  {
    m2->m_matrix=0L;
    delete m2;
  }

  return new Matrix( TMatrix::div(mm1,mm2,managem1,managem2) );
}


Matrix *Matrix::pow(Matrix * &m1, Constant * &c2, bool managem1, bool managem2)
{
  TMatrix *mm1;
  T *cc2;
  mm1=m1->m_matrix;
  cc2=c2->t();
  if (managem1)
  {
    m1->m_matrix=0L;
    delete m1;
  }
  if (managem2)
  {
    delete c2;
  }

  return new Matrix( TMatrix::pow(mm1,cc2,managem1,managem2) );

}

Map *Matrix::simplify(void)
{
  bool nulluptonow=true;
  TMatrix *mat=0L;
  Map *r;
  unsigned int i,j,ii,jj; 
  for (i=0;i<m_matrix->nRows();i++)
    for (j=0;j<m_matrix->nCols();j++)
    {
      r=m_matrix->at(i,j)->simplify();
      if (r)
      {
	if (nulluptonow)
	{
          nulluptonow=false;
	  // We can simplify the matrix, so lets copy the
	  // elements which we didn't simplified earlier
	  mat=new TMatrix(m_matrix->nRows(),m_matrix->nCols());
	  
	  for (ii=0;ii<i;ii++)
	    for (jj=0;jj<m_matrix->nCols();jj++)
	      mat->set(ii,jj,m_matrix->at(ii,jj)->copy());

	  // and the partial row which we were trying
	  for (jj=0;jj<j;jj++)
	      mat->set(i,jj,m_matrix->at(i,jj)->copy());
	}
	
	mat->set(i,j,r);
      } 
      else if (!nulluptonow) 
      {
	// If we've simplified any element then we have to copy the matrix
	mat->set(i,j,m_matrix->at(i,j)->copy());
      }
    }
  
  return (nulluptonow)?0L:(new Matrix(mat));
} 
 
QString Matrix::string(void) const
{
  return m_matrix->string();
}

QString Matrix::htmlString(void) const
{
  return m_matrix->htmlString();
}

QString Matrix::help(void) const
{
  kdDebug() << helpPrefix() << endl;
  if (!m_name.isEmpty())
    return helpPrefix()+QString("<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0><TR><TD><H3><I>%1&nbsp;=&nbsp;</I></H3></TD><TD>%2</TD></TR></TABLE><P>User Matrix<P>")
      .arg(m_name).arg(m_matrix->htmlString())+helpPostfix();
  return helpPrefix()+QString(i18n("No help available"))+helpPostfix();
} 

 
Map *Matrix::derive(const class Variable &var, const QDict <Map> &vars)
{
  TMatrix *mat=new TMatrix(m_matrix->nRows(),m_matrix->nCols());

  for (unsigned int i=0;i<m_matrix->nRows();i++)
    for (unsigned int j=0;j<m_matrix->nCols();j++)
      mat->set(i,j,m_matrix->at(i,j)->derive(var,vars));

  return new Matrix(mat);

}
