/*  fehlberg.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "fehlberg.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include <kdebug.h>
#include <kapp.h>
#include <unistd.h>

MethodFehlberg::MethodFehlberg(void) : MapMethod("MFehlberg")
{
  m_nParameters=5;
  m_parameters=new Parameter *[m_nParameters];

  m_parameters[0]=new Parameter("f");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("y0");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("a");
  m_parameters[2]->setDataType("Constant");
  m_parameters[3]=new Parameter("b");
  m_parameters[3]->setDataType("Constant");
  m_parameters[4]=new Parameter("m");
  m_parameters[4]->setDataType("Constant");
}

MethodFehlberg::MethodFehlberg(const MethodFehlberg &m) : MapMethod(m)
{
}

MethodFehlberg::~MethodFehlberg()
{
}
  
Map *MethodFehlberg::copy(void) const
{
  return new MethodFehlberg(*this);
}

void MethodFehlberg::calcH0(const TMatrix &f,const TMatrix &x, const TMatrix &y, 
    QDict <Map> &vars,unsigned int k, unsigned int n,const T &h, TMatrix &H,
    const T &,const T &,const T &,const T &,const T &,
    TMatrix &fx1, TMatrix &, TMatrix &, TMatrix &, TMatrix &)
{
  Map *fi;
  unsigned int i,j;

  for (i=0;i<n;i++)
  {
    fi=f.at(i);

    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    vars.replace("x",x.at(k-1));
    for (j=0;j<n;j++) 
      vars.replace(QString("y%1").arg(j),y.at(j,k-1));

    fx1.set(i,0,fi->eval(vars));

    H.set(0,i,new Constant(T::add(y.t(i,k-1),&(h*T::newTG(1.0/4)*fx1.tRef(i)))));
  }
}

void MethodFehlberg::calcH1(const TMatrix &f,const TMatrix &, const TMatrix &y, 
    QDict <Map> &vars,unsigned int k, unsigned int n,const T &h, TMatrix &H,
    const T &x2,const T &,const T &,const T &,const T &,
    TMatrix &fx1, TMatrix &fx2, TMatrix &, TMatrix &, TMatrix &)
{
  Map *fi;
  unsigned int i,j;
  
  T &c1=T::newTG(3.0/32);
  T &c2=T::newTG(9.0/32);
  T &t=T::newTG(0.0);
  for (i=0;i<n;i++)
  {
    fi=f.at(i);
    
    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    vars.replace("x",Constant::newConstantG(x2));
    for (j=0;j<n;j++) 
      vars.replace(QString("y%1").arg(j),H.at(0,j));

    fx2.set(i,0,fi->eval(vars));

    t=h*(c1*fx1.tRef(i)+c2*fx2.tRef(i) );
    H.set(1,i,new Constant(T::add(y.t(i,k-1),&t)));
  }

}

void MethodFehlberg::calcH2(const TMatrix &f,const TMatrix &, const TMatrix &y, 
    QDict <Map> &vars,unsigned int k, unsigned int n,const T &h, TMatrix &H,
    const T &,const T &x3,const T &,const T &,const T &,
    TMatrix &fx1, TMatrix &fx2, TMatrix &fx3, TMatrix &, TMatrix &)
{
  Map *fi;
  unsigned int i,j;
  
  T &c1=T::newTG(1932.0/2197);
  T &c2=T::newTG(7200.0/2197);
  T &c3=T::newTG(7296.0/2197);
  T &t=T::newTG(0.0);
  for (i=0;i<n;i++)
  {
    fi=f.at(i);
    
    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    vars.replace("x",Constant::newConstantG(x3));
    for (j=0;j<n;j++) 
      vars.replace(QString("y%1").arg(j),H.at(1,j));
    
    fx3.set(i,0,fi->eval(vars));

    t=h*(c1*fx1.tRef(i)-c2*fx2.tRef(i)+c3*fx3.tRef(i) );
    H.set(2,i,new Constant(T::add(y.t(i,k-1),&t)));
  }

}

void MethodFehlberg::calcH3(const TMatrix &f,const TMatrix &, const TMatrix &y, 
    QDict <Map> &vars,unsigned int k, unsigned int n,const T &h, TMatrix &H,
    const T &,const T &,const T &x4,const T &,const T &,
    TMatrix &fx1, TMatrix &fx2, TMatrix &fx3, TMatrix &fx4, TMatrix &)
{
  Map *fi;
  unsigned int i,j;
  
  T &c1=T::newTG(439.0/216);
  T &c2=T::newTG(8.0);
  T &c3=T::newTG(3680.0/513);
  T &c4=T::newTG(845.0/4104);
  T &t=T::newTG(0.0);
  for (i=0;i<n;i++)
  {
    fi=f.at(i);
    
    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    vars.replace("x",Constant::newConstantG(x4));
    for (j=0;j<n;j++) 
      vars.replace(QString("y%1").arg(j),H.at(2,j));

    fx4.set(i,0,fi->eval(vars));

    t=h*(c1*fx1.tRef(i)-c2*fx2.tRef(i)+c3*fx3.tRef(i)-c4*fx4.tRef(i) );
    H.set(3,i,new Constant(T::add(y.t(i,k-1),&t)));
  }

}

void MethodFehlberg::calcH4(const TMatrix &f,const TMatrix &, const TMatrix &y, 
    QDict <Map> &vars,unsigned int k, unsigned int n,const T &h, TMatrix &H,
    const T &,const T &,const T &,const T &x5,const T &,
    TMatrix &fx1, TMatrix &fx2, TMatrix &fx3, TMatrix &fx4, TMatrix &fx5)
{
  Map *fi;
  unsigned int i,j;
  
  T &c1=T::newTG(-8.0/27);
  T &c2=T::newTG(2.0);
  T &c3=T::newTG(3544.0/2565);
  T &c4=T::newTG(1859.0/4104);
  T &c5=T::newTG(11/40);
  T &t=T::newTG(0.0);
  for (i=0;i<n;i++)
  {
    fi=f.at(i);
    
    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    vars.replace("x",Constant::newConstantG(x5));
    for (j=0;j<n;j++) 
      vars.replace(QString("y%1").arg(j),H.at(3,j));
    
    fx5.set(i,0,fi->eval(vars));

    t=h*(c1*fx1.tRef(i)+c2*fx2.tRef(i)-c3*fx3.tRef(i)+c4*fx4.tRef(i)-c5*fx5.tRef(i) );
    H.set(4,i,new Constant(T::add(y.t(i,k-1),&t)));
  }

}
Map *MethodFehlberg::eval(const QDict <Map> &vars)
{
  kdDebug() << "Fehlberg::eval" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  TMatrix f(*vars2->find("f")->tmatrix());
  
  const TMatrix y0(*vars2->find("y0")->tmatrix());
  T &a=*T::newT(*vars2->find("a")->t());
  T &b=*T::newT(*vars2->find("b")->t());
  unsigned int n=f.nRows();
  int m=(int) *vars2->find("m")->t();
  
  T &tol=*T::newT(*maps.find("_tolerance")->t());
  T &hfactor=*T::newT(*maps.find("_stepControlFactor")->t());
  int maxIter=(int) *maps.find("_maxIterations")->t();
  T &e=*T::newT(0.0);
  bool stepControl=(*maps.find("_stepControl")->t()==e)?false:true;
  
  QDict <Map> tmpvars;

  TMatrix x(1,2);
  TMatrix y(n,2);
  TMatrix z(n,1);
  TMatrix H(5,n);
  unsigned int i,k,j;
  for (i=0;i<n;i++) 
  {
    y.set(i,0,y0.at(i)->copy());
    f.at(i)->propagateForceEvaluation();
  }
  
  T &h=*T::newT((b-a)/(T::newTG(m)-T::newTG(1)));  // Seems a nice first step size
//  T &h=*T::newT((b-a)/(T::newTG(100)));  
  Map *fi;

  T &x2=*T::newT(0.0);
  T &x3=*T::newT(0.0);
  T &x4=*T::newT(0.0);
  T &x5=*T::newT(0.0);
  T &x6=*T::newT(0.0);

  TMatrix fx1(n,1);
  TMatrix fx2(n,1);
  TMatrix fx3(n,1);
  TMatrix fx4(n,1);
  TMatrix fx5(n,1);
  Map *fx6;
  bool increaseH=false;

  x.set(0,0,new Constant(a));

  T &c1=*T::newT(25.0/216);
  T &c2=*T::newT(1408.0/2565);
  T &c3=*T::newT(2197.0/4104);
  T &c4=*T::newT(1.0/5);
  T &cc1=*T::newT(16.0/135);
  T &cc2=*T::newT(6656.0/12825);
  T &cc3=*T::newT(28561.0/56430);
  T &cc4=*T::newT(9.0/50);
  T &cc5=*T::newT(2.0/55);
  T *t=0L;
  kdDebug() << "Fehlberg::eval3\n" << endl;

  k=0;
  while (x.tRef(k)<b)
  {
    k++;
    kdDebug() << "k ( " << k << " )" << endl;
    kdDebug() << "xk ( " << static_cast<double>(x.tRef(k-1)) << " )" << endl;

    bool done=false;
    int it=0;
    while (!done)
    {
      Garbage::push();

      x2=x.tRef(k-1)+T::newTG(1.0/4)*h;
      x3=x.tRef(k-1)+T::newTG(3.0/8)*h;
      x4=x.tRef(k-1)+T::newTG(12.0/13)*h;
      x5=x.tRef(k-1)+h;
      x6=x.tRef(k-1)+T::newTG(1.0/2)*h;

      calcH0(f,x,y,tmpvars,k,n,h,H,x2,x3,x4,x5,x6,fx1,fx2,fx3,fx4,fx5);
      calcH1(f,x,y,tmpvars,k,n,h,H,x2,x3,x4,x5,x6,fx1,fx2,fx3,fx4,fx5);
      calcH2(f,x,y,tmpvars,k,n,h,H,x2,x3,x4,x5,x6,fx1,fx2,fx3,fx4,fx5);
      calcH3(f,x,y,tmpvars,k,n,h,H,x2,x3,x4,x5,x6,fx1,fx2,fx3,fx4,fx5);
      calcH4(f,x,y,tmpvars,k,n,h,H,x2,x3,x4,x5,x6,fx1,fx2,fx3,fx4,fx5);

      for (i=0;i<n;i++)
      {
	fi=f.at(i);

	/**
	 * Now we evaluate f (x   , y     , ..., y     )
	 *                  i  k-1   0,k-1        n,k-1
	 */ 
	tmpvars.clear();

	if (fi->nArgs()==0) 
	{
	  tmpvars.replace("x",Constant::newConstantG(x6));
	  for (j=0;j<n;j++) 
	    tmpvars.replace( QString("y%1").arg(j) , H.at(4,j) );
	} else kdDebug() << " There are arguments !!" << endl;

	fx6=fi->eval(tmpvars);

	t=T::mul(&h,&(c1*fx1.tRef(i)+c2*fx3.tRef(i)+c3*fx4.tRef(i)-c4*fx5.tRef(i)));
	y.set(i,k,new Constant(T::add(y.t(i,k-1),t)));
	delete t;

	if (stepControl)
	{
	  t=T::mul(&h,&(cc1*fx1.tRef(i)+cc2*fx3.tRef(i)+cc3*fx4.tRef(i)-cc4*fx5.tRef(i)+cc5*(*fx6->t()) ));
	  z.set(i,0,new Constant(T::add(y.t(i,k-1),t)));
	  delete t;
	}

	delete fx6;
      }

      if (stepControl)
      {
	e=TMatrix::distMaxCol(&y,k,&z,0);
	
	it++;
	increaseH=false;
	
	if ((e<=tol)||(it>=maxIter)) 
	{
	  if (it<maxIter) // == if (e<=tol)
	  {
	    kdDebug() << QString("h = %1").arg(h.string()) << endl;
	    increaseH=true;
	  }
	  else
	    kdDebug() << QString("Max iterations exceeded : h = %1").arg(h.string()) << endl;

	  done=true;

	}
	else 
	{
	  kdDebug() << "Let's do another iteration with a smaller h" << endl;
	  h*=((tol/e).sqrt(4));
	  h*=T::newTG(0.9);
	}
      }
      else  // the step control is disabled
	done=true;

      Garbage::pop();
//      kapp->processEvents(200);
    }

    x.set(0,k,new Constant(T::add(x.t(k-1),&h)));

    if (increaseH) h*=hfactor;

  }
  
  kdDebug() << "Fehlberg::eval(end)" << endl;
  delete vars2;
  
  TMatrix *tr=new TMatrix(1,2);
  tr->set(0,0, new Matrix(x));
  tr->set(0,1, new Matrix(y));
  
  T::deleteT(c4);
  T::deleteT(c3);
  T::deleteT(c2);
  T::deleteT(c1);
  T::deleteT(cc5);
  T::deleteT(cc4);
  T::deleteT(cc3);
  T::deleteT(cc2);
  T::deleteT(cc1);
 
  T::deleteT(a);
  T::deleteT(b);
  T::deleteT(e);
  T::deleteT(x2);
  T::deleteT(x3);
  T::deleteT(x4);
  T::deleteT(x5);
  T::deleteT(x6);
  T::deleteT(hfactor);
  T::deleteT(tol);

  kdDebug() << "Fehlberg::eval(end2)" << endl;
  return new Matrix(tr);
}

QString MethodFehlberg::string(void) const
{
  return "MethodFehlberg";
}

QString MethodFehlberg::help(void) const
{
  return helpPrefix()+"<H3><I>MFehlberg( F, y0, a, b, M )</I></H3><BR><BR>Solves a system of "
    "differential equations using the Fehlberg method (with step control)"+helpPostfix();
}
