/*  fehlberg.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "fehlberg78.h"
#include "../parameter.h"
#include "../constant.h"
#include "../matrix.h"
#include "../tmatrix.h"
#include "../vartype.h"
#include <kdebug.h>
#include <kstddirs.h>
#include <kapp.h>
#include <unistd.h>

MethodFehlberg78::MethodFehlberg78(void) : MapMethod("MFehlberg78")
{
  m_nParameters=5;
  m_parameters=new Parameter *[m_nParameters];

  m_parameters[0]=new Parameter("f");
  m_parameters[0]->setDataType("Matrix");
  m_parameters[1]=new Parameter("y0");
  m_parameters[1]->setDataType("Matrix");
  m_parameters[2]=new Parameter("a");
  m_parameters[2]->setDataType("Constant");
  m_parameters[3]=new Parameter("b");
  m_parameters[3]->setDataType("Constant");
  m_parameters[4]=new Parameter("m");
  m_parameters[4]->setDataType("Constant");
}

MethodFehlberg78::MethodFehlberg78(const MethodFehlberg78 &m) : MapMethod(m)
{
}

MethodFehlberg78::~MethodFehlberg78()
{
}
  
Map *MethodFehlberg78::copy(void) const
{
  return new MethodFehlberg78(*this);
}

void MethodFehlberg78::calcHi(const TMatrix &f,const TMatrix &x, 
    const TMatrix &y, QDict <Map> &vars, unsigned int k, unsigned int n, 
    unsigned int Hi, const T &h, TMatrix &H, const TMatrix &xx, TMatrix &fx, 
    const TMatrix &coef)
{
  Map *fi;
  unsigned int i,j,l;

  T &t=T::newTG(0.0);
  for (i=0;i<n;i++)
  {
    fi=f.at(i);

    if (Hi==0)
      vars.replace("x",x.at(0));
    else
      vars.replace("x",xx.at(Hi));

    /**
     * Now we evaluate f (x   , y     , ..., y     )
     *                  i  k-1   0,k-1        n,k-1
     */ 

    if (Hi==0)
      for (j=0;j<n;j++) 
	vars.replace(QString("y%1").arg(j),y.at(j,k-1));
    else
      for (j=0;j<n;j++) 
	vars.replace(QString("y%1").arg(j),H.at(Hi-1,j));

    fx.set(i,Hi,fi->eval(vars));

    t=coef.tRef(Hi,0)*fx.tRef(i,0);
    for (l=0;l<Hi;l++)
      t+=coef.tRef(Hi,l+1)*fx.tRef(i,l);

    H.set(Hi,i,new Constant(T::add(y.t(i,k-1),&(h*t))));
  }
}

Map *MethodFehlberg78::eval(const QDict <Map> &vars)
{
  kdDebug() << "Fehlberg78::eval" << endl;
  QDict <Map> *vars2=evalParams(vars);
  if (!checkParamTypes(*vars2)) return 0L;
  TMatrix f(*vars2->find("f")->tmatrix());
  
  const TMatrix y0(*vars2->find("y0")->tmatrix());
  T &a=*T::newT(*vars2->find("a")->t());
  T &b=*T::newT(*vars2->find("b")->t());
  unsigned int n=f.nRows();
  int m=(int) *vars2->find("m")->t();
  
  T &tol=*T::newT(*maps.find("_tolerance")->t());
  T &hfactor=*T::newT(*maps.find("_stepControlFactor")->t());
  int maxIter=(int) *maps.find("_maxIterations")->t();
  T &e=*T::newT(0.0);
  bool stepControl=(*maps.find("_stepControl")->t()==e)?false:true;
  
  QDict <Map> tmpvars;

  TMatrix x(1,2);
  TMatrix y(n,2);
  TMatrix z(n,1);
  TMatrix H(14,n);
  unsigned int i,k,j;
  for (i=0;i<n;i++) 
  {
    y.set(i,0,y0.at(i)->copy());
    f.at(i)->propagateForceEvaluation();
  }
  
  T &h=*T::newT((b-a)/(T::newTG(m)-T::newTG(1)));  // Seems a nice first step size
//  T &h=*T::newT((b-a)/(T::newTG(100)));  
  Map *fi;

  TMatrix fx(n,13);
  bool increaseH=false;

  x.set(0,0,new Constant(a));

  T &c1=*T::newT(41.0/840);
  T &c2=*T::newT(34.0/105);
  T &c3=*T::newT(9.0/35);
  T &c4=*T::newT(9.0/280);
  T &cc1=*T::newT(34.0/105);
  T &cc2=*T::newT(9.0/35);
  T &cc3=*T::newT(9.0/280);
  T &cc4=*T::newT(41.0/840);
  T *t=0L;
  kdDebug() << "Fehlberg78::eval3\n" << endl;

  TMatrix *coef=new TMatrix(1,1);
  kdDebug() << "load matrix" << endl;
  coef->load(locate("appdata","coefficients/fehlberg78"));
  kdDebug() << "matrix loaded" << endl;
  kdDebug() << coef->string() << endl;

  k=0;
  TMatrix *xx=new TMatrix(13,1);
  while (x.tRef(k)<b)
  {
    Garbage::push();
    k++;
    kdDebug() << "k ( " << k << " )" << endl;
    kdDebug() << "xk ( " << static_cast<double>(x.tRef(k-1)) << " )" << endl;

    bool done=false;
    int it=0;
    while (!done)
    {
      Garbage::push();

      xx->set(0,0,x.at(k-1)->copy());
      xx->set(1,0,new Constant(x.tRef(k-1)+T::newTG(2.0/27)*h));
      xx->set(2,0,new Constant(x.tRef(k-1)+T::newTG(1.0/9)*h));
      xx->set(3,0,new Constant(x.tRef(k-1)+T::newTG(1.0/6)*h));
      xx->set(4,0,new Constant(x.tRef(k-1)+T::newTG(5.0/12)*h));
      xx->set(5,0,new Constant(x.tRef(k-1)+T::newTG(1.0/2)*h));
      xx->set(6,0,new Constant(x.tRef(k-1)+T::newTG(5.0/6)*h));
      xx->set(7,0,new Constant(x.tRef(k-1)+T::newTG(1.0/6)*h));
      xx->set(8,0,new Constant(x.tRef(k-1)+T::newTG(2.0/3)*h));
      xx->set(9,0,new Constant(x.tRef(k-1)+T::newTG(1.0/3)*h));
      xx->set(10,0,new Constant(x.tRef(k-1)+h));
      xx->set(11,0,new Constant(x.tRef(k-1)));
      xx->set(12,0,new Constant(x.tRef(k-1)+h));

      for (int l=0;l<12;l++)
	calcHi(f,x,y,tmpvars,k,n,l,h,H,*xx,fx,*coef);

      for (i=0;i<n;i++)
      {
	fi=f.at(i);

	/**
	 * Now we evaluate f (x   , y     , ..., y     )
	 *                  i  k-1   0,k-1        n,k-1
	 */ 
	tmpvars.clear();

	tmpvars.replace("x",xx->at(12,0));
	for (j=0;j<n;j++) 
	  tmpvars.replace( QString("y%1").arg(j) , H.at(11,j) );

	fx.set(i,12,fi->eval(tmpvars));

//	t=T::mul(&h,&(c1*fx1.tRef(i)+c2*fx3.tRef(i)+c3*fx4.tRef(i)-c4*fx5.tRef(i)));
	t=T::mul(&h,&(c1*fx.tRef(i,0)+c2*fx.tRef(i,5)+c3*fx.tRef(i,6)+c3*fx.tRef(i,7)+c4*fx.tRef(i,8)+c4*fx.tRef(i,9)+c1*fx.tRef(i,10)));
	y.set(i,k,new Constant(T::add(y.t(i,k-1),t)));
	delete t;

	if (stepControl)
	{
//	  t=T::mul(&h,&(cc1*fx1.tRef(i)+cc2*fx3.tRef(i)+cc3*fx4.tRef(i)-cc4*fx5.tRef(i)+cc5*(*fx6->t()) ));
	  t=T::mul(&h,&(cc1*fx.tRef(i,5)+cc2*fx.tRef(i,6)+cc2*fx.tRef(i,7)+cc3*fx.tRef(i,8)+cc3*fx.tRef(i,9)+cc4*fx.tRef(i,11)+cc4*fx.tRef(i,12)));
	  z.set(i,0,new Constant(T::add(y.t(i,k-1),t)));
	  delete t;
	}

//	delete fx6;
      }

      if (stepControl)
      {
	e=TMatrix::distMaxCol(&y,k,&z,0);
	
	it++;
	increaseH=false;
	
	if ((e<=tol)||(it>=maxIter)) 
	{
	  if (it<maxIter) // == if (e<=tol)
	  {
	    kdDebug() << QString("h = %1").arg(h.string()) << endl;
	    increaseH=true;
	  }
	  else
	    kdDebug() << QString("Max iterations exceeded : h = %1").arg(h.string()) << endl;

	  done=true;

	}
	else 
	{
	  kdDebug() << "Let's do another iteration with a smaller h" << endl;
	  h*=((tol/e).sqrt(4));
	  h*=T::newTG(0.9);
	}
      }
      else  // the step control is disabled
	done=true;

      Garbage::pop();
//      kapp->processEvents(200);
    }

    x.set(0,k,new Constant(T::add(x.t(k-1),&h)));

    if (increaseH) h*=hfactor;

    Garbage::pop();
  }
  
  kdDebug() << "Fehlberg78::eval(end)" << endl;
  delete vars2;
  
  TMatrix *tr=new TMatrix(1,2);
  tr->set(0,0, new Matrix(x));
  tr->set(0,1, new Matrix(y));
  
  T::deleteT(c4);
  T::deleteT(c3);
  T::deleteT(c2);
  T::deleteT(c1);
  T::deleteT(cc4);
  T::deleteT(cc3);
  T::deleteT(cc2);
  T::deleteT(cc1);
 
  T::deleteT(a);
  T::deleteT(b);
  T::deleteT(e);
  T::deleteT(hfactor);
  T::deleteT(tol);

  kdDebug() << "Fehlberg78::eval(end2)" << endl;
  return new Matrix(tr);
}

QString MethodFehlberg78::string(void) const
{
  return "MethodFehlberg78";
}

QString MethodFehlberg78::help(void) const
{
  return helpPrefix()+"<H3><I>MFehlberg78( F, y0, a, b, M )</I></H3><BR><BR>Solves a system of "
    "differential equations using the Fehlberg78 method (with step control)"+helpPostfix();
}
