/*  mextract.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "mextract.h"
#include "constant.h"
#include "matrix.h"
#include "tmatrix.h"
#include "parameter.h"
#include <kdebug.h>

MatrixExtract::MatrixExtract(const QString &decl) : MapVariableArguments(decl)
{
  m_isAClassName="MatrixExtract";
}

MatrixExtract::MatrixExtract(const MatrixExtract &m) : MapVariableArguments( m )
{
  m_isAClassName="MatrixExtract";
}

MatrixExtract::MatrixExtract(Map *m) : MapVariableArguments( )
{
  m_isAClassName="MatrixExtract";
  m_tree=m;
}
  
Map *MatrixExtract::copy(void) const
{
  return new MatrixExtract(*this);
}

Map *MatrixExtract::eval(const QDict <Map> &vars)
{
  if (nArgs()==0) return new Constant(0.0); // XXX
  kdDebug() << "MatrixExtract::eval" << endl;

   // This could be optimized to evaluate only the part of the matrix
   // we're going to extract, but it's OK this way for now.
   m_tree->setForceEvaluation(true);
   Map *prer=m_tree->eval(vars);
   Map *r=0L;

   if (prer->isA("Matrix"))
   {
     QDict <Map> *vars2=evalParams(vars);
     TMatrix *matrix=prer->tmatrix();
     if (nArgs()==1)
     {
       Map *p1=vars2->find("$0");
       if (!p1->isA("Constant"))
       {
/*	 r=new MatrixExtract(prer);
	 r->m_tree=prer->copy(); // prer is deleted some lines below
*/
	 r=new MatrixExtract(prer->copy()); // prer is deleted some lines below
	 for (unsigned int i=0;i<m_nParameters;i++)
	   r->setParameterExpression(i,vars2->find(m_parameters[i]->name())->copy());

       }
       else
       {
	 T *t1=p1->t();
	 if (matrix->nRows()==1)  // It's a row vector
	 {
	   r=matrix->at(0,(int)*t1);
	   if (r) r=r->copy();
	 }
	 else if (matrix->nCols()==1) // It's a column vector
	 {
	   r=matrix->at((int)*t1,0);
	   if (r) r=r->copy();
	 }
	 else  // We have to extract the t1-th row
	 {  
	   TMatrix *rmatrix=new TMatrix(1,matrix->nCols());

	   unsigned int j=static_cast<int>(*t1);
	   for (unsigned int i=0;i<matrix->nCols();i++) rmatrix->set(0,i,matrix->at(j,i)->copy());

	   r=new Matrix(rmatrix);
	 }
       }
     }
     else if (nArgs()==2)
     {
       QDict <Map> *vars2=evalParams(vars);
       Map *p1=vars2->find("$0");
       Map *p2=vars2->find("$1");
       if ((!p1->isA("Constant"))||(!p2->isA("Constant"))) 
       {
	 r=new MatrixExtract(prer->copy()); // prer is deleted some lines below
/*	 r=new MatrixExtract(prer);
	 r->m_tree=prer->copy(); // prer is deleted some lines below
*/
	 for (unsigned int i=0;i<m_nParameters;i++)
	   r->setParameterExpression(i,vars2->find(m_parameters[i]->name())->copy());
       }
       else
       {
	 T *t1=p1->t();
	 T *t2=p2->t();
	 r=matrix->at((int)*t1,(int)*t2);
	 if (r) r=r->copy();
       }
     }

     delete vars2;
     delete prer;

   } else 
   {
     QDict <Map> *vars2=evalParams(vars);
     r=new MatrixExtract(prer);
/*     r=new MatrixExtract(prer);
     r->m_tree=prer;
*/
     for (unsigned int i=0;i<m_nParameters;i++)
       r->setParameterExpression(i,vars2->find(m_parameters[i]->name())->copy());

     delete vars2;
   }

   return r;
  
};

QString MatrixExtract::string(void) const
{
  if (!m_tree) return QString("MatrixExtract");
  if (nArgs()==2) return QString("%1[ %2, %3 ]").arg(m_tree->string()).
    arg(argument(0)->string()).arg(argument(1)->string());
  else if (nArgs()==1) return QString("%1[ %2 ]").arg(m_tree->string()).arg(m_parameters[0]->string());
  else return QString("%1").arg(m_tree->string());
}
  
