/*  parameter.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "parameter.h"
#include "variable.h"
#include <kdebug.h>

Parameter::Parameter(const QString &nam, const QString &def)
{
  pName=nam;
  prefDataType=0L;
//  kdDebug() << QString("new Parameter (%1) <- (%2)").arg(nam).arg(def) << endl;

  if (/*(pName==def)||*/(def.isEmpty()))
  {
//    tree=0L;
    tree=new Variable(pName);
  }
  else
  {
    tree=Map::parse(def);
  }
}

Parameter::Parameter(const Parameter &par)
{
  pName=par.pName;

  if (par.prefDataType) prefDataType=new QString(*par.prefDataType);
  else prefDataType=0L;
  
  if (par.tree) tree=par.tree->copy();
  else tree=0L;
}

Parameter::Parameter(const QString &nam, Map *def)
{
  pName=nam;
  prefDataType=0L;
  tree=def;
//  kdDebug() << QString("new2 Parameter %1 <- %2").arg(pName).arg(tree->string()) << endl;
}

Parameter::~Parameter()
{
  delete tree;
  delete prefDataType;
}

void Parameter::setName(const QString & nam)
{
  pName=nam;
}

/*
Parameter &Parameter::operator =(const Parameter &p)
{
  kdDebug() << "Parameter::=" << endl;
  kdDebug() << pName << endl;
  pName=p.pName;

  delete prefDataType;
  if (p.prefDataType) prefDataType=new QString(*p.prefDataType);
  else prefDataType=0L;
  kdDebug() << "Parameter::=2" << endl;

  kdDebug() << tree->string() << endl;
  kdDebug() << tree->isA() << endl;
  delete tree;
  if (p.tree)
  {
    tree=p.tree->copy();
  }
  else tree=0L;
  kdDebug() << "Parameter::=3" << endl;

  return *this;
}
*/
 
void Parameter::setDataType(const QString & pref)
{
  delete prefDataType;
  prefDataType=new QString(pref);
}

int Parameter::checkDataType(const QDict <Map> & vars) const
{
  if ((!prefDataType)||(prefDataType->isEmpty())) return true;

  Map *m=vars.find(pName);
  if (m==0L) return true;

  return m->isA(*prefDataType);
}

Map *Parameter::eval(const QDict <Map> &vars)
{
  Map *r;
  if (tree)
  {
    QDict <Map> vars2(vars);
    r=tree->eval(vars2);
  }
  else
  {
    Map *t=vars.find(pName);
    if (t) r=t->copy();
    else
    {
      kdDebug() << QString("ERROR: Parameter::eval : No variable named (%1)").arg(pName) << endl;
      r=new Variable(pName);
    }
  } 
  
  return r;
}

Parameter *Parameter::copy() const
{
  return new Parameter(*this);
}

void Parameter::printDebug(const QString &tab) const
{
  kdDebug() << tab+"Parameter "+pName << endl;
  if (tree!=0L) tree->printDebug(tab+"\\-> ");
  else kdDebug() << tab+"no tree" << endl;
  
}

QString Parameter::string(void) const
{
  if (tree) return tree->string();
  return pName;
}
