/*  vargmp.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "vargmp.h"
#include <qstring.h>
#include <kdebug.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "garbage.h"

#define PRECISION 65536

void mpf_pow(mpf_ptr r, mpf_srcptr x, mpf_srcptr y);

TArbitraryPrecision::TArbitraryPrecision(const mpf_t & g) : T (0)
{
  mpf_init2(v,PRECISION);
  mpf_set(v,g);
}

TArbitraryPrecision::TArbitraryPrecision(const double g) : T(g)
{
  mpf_init2(v,PRECISION);
  mpf_set_d(v,g);
}

TArbitraryPrecision::TArbitraryPrecision(const int g) : T(g)
{
  mpf_init2(v,PRECISION);
  mpf_set_si(v,g);
}

TArbitraryPrecision::TArbitraryPrecision(const unsigned int g) : T(g)
{
  mpf_init2(v,PRECISION);
  mpf_set_ui(v,g);
}

TArbitraryPrecision::TArbitraryPrecision(const QString &s) : T(0)
{
  mpf_init2(v,PRECISION);
  mpf_set_str(v,s.latin1(),10);
}                                                                                

TArbitraryPrecision::TArbitraryPrecision( const TArbitraryPrecision &x) : T (x)
{
  mpf_init2(v,PRECISION);
  mpf_set(v,x.v);
}

TArbitraryPrecision *TArbitraryPrecision::copy(void) const
{
  return new TArbitraryPrecision(*this);
}

inline int TArbitraryPrecision::operator==( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)==0;
}

inline int TArbitraryPrecision::operator==( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)==0;
}

inline int TArbitraryPrecision::operator==( const mpf_t & g )
{
  return mpf_cmp(v,g)==0;
}

inline int TArbitraryPrecision::operator!=( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)!=0;
}

inline int TArbitraryPrecision::operator!=( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)!=0;
}

inline int TArbitraryPrecision::operator!=( const mpf_t & g )
{
  return mpf_cmp(v,g)!=0;
}

inline int TArbitraryPrecision::operator<( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)<0;
}

inline int TArbitraryPrecision::operator<( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)<0;
}

inline int TArbitraryPrecision::operator<( const mpf_t & g )
{
  return mpf_cmp(v,g)<0;
}

inline int TArbitraryPrecision::operator>( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)>0;
}

inline int TArbitraryPrecision::operator>( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)>0;
}

inline int TArbitraryPrecision::operator>( const mpf_t & g )
{
  return mpf_cmp(v,g)>0;
}

inline int TArbitraryPrecision::operator>=( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)>=0;
}

inline int TArbitraryPrecision::operator>=( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)>=0;
}

inline int TArbitraryPrecision::operator>=( const mpf_t & g )
{
  return mpf_cmp(v,g)>=0;
}

inline int TArbitraryPrecision::operator<=( const TArbitraryPrecision &v1 )
{
  return mpf_cmp(v,v1.v)<=0;
}

inline int TArbitraryPrecision::operator<=( const T &v1 )
{
  return mpf_cmp(v,((TArbitraryPrecision &)v1).v)<=0;
}

inline int TArbitraryPrecision::operator<=( const mpf_t & g )
{
  return mpf_cmp(v,g)<=0;
}

T &TArbitraryPrecision::operator=( const TArbitraryPrecision &x )
{
  mpf_set_prec(v,mpf_get_prec(x.v));
  mpf_set(v,x.v);
  return *this;
}

T &TArbitraryPrecision::operator=( const mpf_t & x )
{
  mpf_set_prec(v,mpf_get_prec(x));
  mpf_set(v,x);
  return *this;
}

T &TArbitraryPrecision::operator+=( const TArbitraryPrecision &x )
{
  mpf_add(v,v,x.v);
  return *this;
}

T &TArbitraryPrecision::operator-=( const TArbitraryPrecision &x )
{
  mpf_sub(v,v,x.v);
  return *this;
}
T &TArbitraryPrecision::operator*=( const TArbitraryPrecision &x )
{
  mpf_mul(v,v,x.v);
  return *this;
}
T &TArbitraryPrecision::operator/=( const TArbitraryPrecision &x )
{
  mpf_div(v,v,x.v);
  return *this;
}

T &TArbitraryPrecision::operator+=( const unsigned int i )
{
  mpf_add_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator-=( const unsigned int i )
{
  mpf_sub_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator*=( const unsigned int i )
{
  mpf_mul_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator/=( const unsigned int i )
{
  mpf_div_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator+=( const int i )
{
  if (i<0) mpf_sub_ui(v,v,-i);
  else mpf_add_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator-=( const int i )
{
  if (i<0) mpf_add_ui(v,v,-i);
  else mpf_sub_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator*=( const int i )
{
  if (i<0) 
  {
    mpf_mul_ui(v,v,(unsigned int)-i);
    mpf_neg(v,v);
  }
  else mpf_mul_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::operator/=( const int i )
{
  if (i<0) 
  {
    mpf_div_ui(v,v,(unsigned int)-i); 
    mpf_neg(v,v);
  }
  else mpf_div_ui(v,v,i);
  return *this;
}

T &TArbitraryPrecision::abs(void)
{
  mpf_abs(v,v);
  return *this;
}

T &TArbitraryPrecision::sqrt(const double &idx)
{
  if (idx==2.0)
    mpf_sqrt(v,v);
  else
    mpf_set_d(v,::pow( mpf_get_d(v), 1.0/idx ));

  return *this;
}

T &TArbitraryPrecision::sin( void )
{
  mpf_set_d(v,::sin( mpf_get_d(v) ));
  return *this;
}
 
T &TArbitraryPrecision::cos( void )
{
  mpf_set_d(v,::cos( mpf_get_d(v) ));
  return *this;
}
 
T &TArbitraryPrecision::tan( void )
{
  mpf_set_d(v,::tan( mpf_get_d(v) ));
  return *this;
} 

T &TArbitraryPrecision::exp( void )
{
  mpf_set_d(v,::exp( mpf_get_d(v) ));
  return *this;
}

T &TArbitraryPrecision::log( void )
{
  mpf_set_d(v,::log( mpf_get_d(v) ));
  return *this;
}

T &TArbitraryPrecision::log( const T &base )
{
  mpf_t t;
  mpf_init_set_d (v,::log( mpf_get_d(v)));
  mpf_init_set_d (t,::log( mpf_get_d(((TArbitraryPrecision *)&base)->value())));
  mpf_div(v, v , t);
  return *this;
}

/*TArbitraryPrecision TArbitraryPrecision::operator-( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  TArbitraryPrecision tmp( v1 );
  tmp-=v2;
  return tmp;
}

TArbitraryPrecision TArbitraryPrecision::operator*( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  TArbitraryPrecision tmp( v1 );
  tmp*=v2;
  return tmp;
}

TArbitraryPrecision TArbitraryPrecision::operator/( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  TArbitraryPrecision tmp( v1 );
  tmp/=v2;
  return tmp;
}
*/
QString TArbitraryPrecision::string(void) const
{
/*  char s[4935];
  sprintf(s,"%.4932Lg",v);
  return QString(s);
  */
//  kdDebug() << "TArbitraryPrecision::string" << endl;
  mp_exp_t exp;
  char *t=mpf_get_str(NULL,&exp,10,0,v);
  unsigned long exponent=exp;
//  kdDebug() << "TArbitraryPrecision::string_1 " << exponent << "d " << strlen(t) << " (" << t << ")" << endl;
  if (t[0]=='-') exponent++;

//  int decimalNumbers=32;
  if (strlen(t)-exponent>m_decimalNumbers)
  {
    free(t);  
    t=mpf_get_str(NULL,&exp,10,exponent+1+m_decimalNumbers,v);
    exponent=exp;
//    kdDebug() << "TArbitraryPrecision::string_1 " << exponent << "d " << strlen(t) << " (" << t << ")" << endl;
    if (t[0]=='-') exponent++;
    

  }
  char *s=new char[((exponent>strlen(t))?exponent:strlen(t))+3];
  char *s2=0L,*t2=0L;

  unsigned long i=0;

    
  if (exponent<strlen(t))
  {
//  kdDebug() << "TArbitraryPrecision::string_1 " << exponent << "d " << strlen(t) << endl;
    for ( i=0, s2=s, t2=t ;i<exponent; i++, s2++, t2++) *s2=*t2;
    *s2='.';
    s2++;
    for ( i=exponent ;i<(strlen(t)); i++, s2++, t2++) *s2=*t2;
    *s2=0;
  }
  else
  {
//  kdDebug() << "TArbitraryPrecision::string_2" << endl;
    for ( i=0, s2=s, t2=t ;i<strlen(t); i++, s2++, t2++) *s2=*t2;
    for ( i=strlen(t) ; i<exponent; i++, s2++) *s2='0';
    *s2=0;
  }
//  kdDebug() << "TArbitraryPrecision::string2" << endl;
  free(t);
//  kdDebug() << "TArbitraryPrecision::string3" << endl;
  QString string=QString::fromLatin1(s);
  delete s;
  return string;
}

TArbitraryPrecision::operator const double () const
{
  return mpf_get_d(v);
}

T &TArbitraryPrecision::operator++( )
{
  mpf_add_ui(v,v,1);
  return *this;
}

T &TArbitraryPrecision::operator++( int )
{
  T *t=copy();
  Garbage::throwT(t);
  mpf_add_ui(v,v,1);
  return *t;
}

T &TArbitraryPrecision::operator--( )
{
  mpf_sub_ui(v,v,1);
  return *this;
}

T &TArbitraryPrecision::operator--( int )
{
  T *t=copy();
  Garbage::throwT(t);
  mpf_sub_ui(v,v,1);
  return *t;
} 
void mpf_pow(mpf_ptr r, mpf_srcptr x, mpf_srcptr y)
{
  mpf_t xx;
  mpf_init(xx);
  mpf_set( xx , x );

  mpz_t i;
  mpz_init(i);
  mpz_set_f( i, y );

  mpf_set( r , x );

  unsigned long k;

  while (mpz_cmp_ui ( i , 1000000L ) >= 0)
  {
    for (k=1000000L;k>0;k--)
    {
      mpf_mul(r,r,xx);
    }
    mpz_sub_ui(i,i,1000000L);
  };

  for (k=mpz_get_ui(i) ;k>0L;k--)
  {
    mpf_mul(r,r,xx);
  }                                                                                               
}

T &operator+( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  T *tmp=TArbitraryPrecision::add(&v1, &v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator-( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  T *tmp=TArbitraryPrecision::sub(&v1, &v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator*( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  T *tmp=TArbitraryPrecision::mul(&v1, &v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator/( const TArbitraryPrecision &v1, const TArbitraryPrecision &v2 )
{
  T *tmp=TArbitraryPrecision::div(&v1, &v2);
  Garbage::throwT(tmp);
  return *tmp;
} 

T *TArbitraryPrecision::add(const TArbitraryPrecision *t1, const TArbitraryPrecision *t2)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp+=*t2;
  return tmp;
}

T *TArbitraryPrecision::sub(const TArbitraryPrecision *t1, const TArbitraryPrecision *t2)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp-=*t2;
  return tmp;
}

T *TArbitraryPrecision::mul(const TArbitraryPrecision *t1, const TArbitraryPrecision *t2)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp*=*t2;
  return tmp;
}

T *TArbitraryPrecision::div(const TArbitraryPrecision *t1, const TArbitraryPrecision *t2)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp/=*t2;
  return tmp;
}

T *TArbitraryPrecision::pow(const TArbitraryPrecision *t1, const TArbitraryPrecision *)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  // TODO 
  return tmp;
}

T *TArbitraryPrecision::add(const TArbitraryPrecision *t1, const unsigned int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp+=i;
  return tmp;
}

T *TArbitraryPrecision::sub(const TArbitraryPrecision *t1, const unsigned int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp-=i;
  return tmp;
}

T *TArbitraryPrecision::mul(const TArbitraryPrecision *t1, const unsigned int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp*=i;
  return tmp;
}

T *TArbitraryPrecision::div(const TArbitraryPrecision *t1, const unsigned int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp/=i;
  return tmp;
}

T *TArbitraryPrecision::add(const TArbitraryPrecision *t1, const int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp+=i;
  return tmp;
}

T *TArbitraryPrecision::sub(const TArbitraryPrecision *t1, const int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp-=i;
  return tmp;
}

T *TArbitraryPrecision::mul(const TArbitraryPrecision *t1, const int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp*=i;
  return tmp;
}

T *TArbitraryPrecision::div(const TArbitraryPrecision *t1, const int i)
{
  TArbitraryPrecision *tmp=new TArbitraryPrecision( *t1 );
  *tmp/=i;
  return tmp;
} 
