/*  variable.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "variable.h"
#include "mapvararg.h"
#include "constant.h"
#include <kdebug.h>

Variable::Variable(const QString &name,bool nosubstitute) : Map()
{
  m_isAClassName="Variable";
  m_name=name;
  m_noSubstitute=nosubstitute;
}

Variable::Variable(const Variable &v) : Map(v)
{
  m_name=v.m_name;
  m_noSubstitute=v.m_noSubstitute;
}

Map *Variable::copy(void) const
{
  return new Variable(*this);
}

Map *Variable::eval (const QDict <Map> &vars)
{
  Map *r;
  Map *r2;
  if (m_noSubstitute) 
  {
    kdDebug() << "Variable::eval : noSubstitute (return copy())" << endl;
    return copy();
  }

  MapVariableArguments *nosubs=static_cast<MapVariableArguments *>(vars.find("$$__noSubstitute"));
  QDict <Map> vars2(vars);
  bool deletenosubs=false;
  if (!nosubs)
  {
    nosubs=new MapVariableArguments();
    nosubs->setName("$$__noSubstitute",false);
    vars2.insert(QString("$$__noSubstitute"),nosubs);
    deletenosubs=true;
  }
  else
  {
    if (nosubs->isArgumentName(m_name))
    {
      kdError() << QString("Recursive Variable evaluation (%1)!!").arg(m_name) << endl;
      return copy();
    }
  }
  nosubs->addParameterName(m_name);
//  kdDebug() << QString("Variable::eval <------------------------------- nosubs= %1").arg(name) << endl;
  
  Map *t=vars.find(m_name);
  if (t) r=t->eval(vars2);
  else if ((r2=functionFactory(m_name))!=0L)
  {
    r=r2->eval(vars2);
    delete r2;
  }
  else
  {
    Variable *v=static_cast<Variable *>(copy());
    
    v->setAllowSubstitution(false);
    r=v;
  }

  nosubs->delParameterName(m_name);
  if (deletenosubs) delete nosubs;
  
  return r;
}

QString Variable::string(void) const
{
#ifdef DEBUGSTRINGS
  return QString("Variable(%1)").arg(m_name);
#else
  return m_name;
#endif
}

void Variable::propagateForceEvaluation( bool b )
{
  m_noSubstitute=(b)?false:true;
}
  
Map *Variable::derive(const Variable &var, const QDict <Map> &)
{
  if (m_name==var.name()) return new Constant(1.0);
  else return new Constant(0.0); // What about implicit functions ?
  // Like writing f(x,y,z) when meaning f(x(t),y(t),z(t)) ?
  // Is there any solution for this ?
}

