/*  vartype.cpp - 
    This file is part of Kalamaris
    Copyright (C) 2000  Antonio Larrosa Jimenez
    Kalamaris' homepage : http://devel-home.kde.org/~larrosa/kalamaris.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "vartype.h"
#include <qstring.h>
#include <stdio.h>
#include <math.h>
#include <kdebug.h>
#include "vargmp.h"
#include "garbage.h"

T::T(const T_DOUBLE g)
{
  v=g;
}

T::T(const int g)
{
  v=g;
}

T::T(const unsigned int g)
{
  v=g;
}

T::T(const QString &s)
{
  v=s.toDouble();
}

T::T( const T &x)
{
  v=x.v;
}

T *T::copy(void) const
{
  return new T(*this);
}

inline int T::operator==( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)==(TArbitraryPrecision &)v1;
  return v==v1.v;
}
  
inline int T::operator==( CT_TYPEREF g )
{
  return v==g;
}

inline int T::operator!=( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)!=(TArbitraryPrecision &)v1;
  return v!=v1.v;
}

inline int T::operator!=( CT_TYPEREF g )
{
  return v!=g;
}

inline int T::operator<( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)<(TArbitraryPrecision &)v1;
  return v<v1.v;
}

inline int T::operator<( CT_TYPEREF g )
{
  return v<g;
}

inline int T::operator>( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)>(TArbitraryPrecision &)v1;
  return v>v1.v;
}

inline int T::operator>( CT_TYPEREF g )
{
  return v>g;
}

inline int T::operator>=( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)>=(TArbitraryPrecision &)v1;
  return v>=v1.v;
}

inline int T::operator>=( CT_TYPEREF g )
{
  return v>=g;
}

inline int T::operator<=( const T &v1 )
{
  if (v1.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)<=(TArbitraryPrecision &)v1;
  return v<=v1.v;
}

inline int T::operator<=( CT_TYPEREF g )
{
  return v<=g;
}

T &T::operator=( const T &x )
{
  if (x.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)=(TArbitraryPrecision &)x;
  v=x.v;
  return *this;
}

T &T::operator=( CT_TYPEREF x )
{
  v=x;
  return *this;
}

T &T::operator+=( const T &x )
{
  if (x.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)+=(TArbitraryPrecision &)x;
  v+=x.v;
  return *this;
}

T &T::operator-=( const T &x )
{
  if (x.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)-=(TArbitraryPrecision &)x;
  v-=x.v;
  return *this;
}
T &T::operator*=( const T &x )
{
  if (x.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)*=(TArbitraryPrecision &)x;
  v*=x.v;
  return *this;
}
T &T::operator/=( const T &x )
{
  if (x.isA("TArbitraryPrecision")) 
    return (*(TArbitraryPrecision *)this)/=(TArbitraryPrecision &)x;
  v/=x.v;
  return *this;
}

T &T::operator+=( const unsigned int i )
{
  if (isA("TArbitraryPrecision")) // Is this check really needed ? 
    // (it's virtual after all)
    return (*(TArbitraryPrecision *)this)+=i;
  v+=i;
  return *this;
}

T &T::operator-=( const unsigned int i )
{
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)-=i;
  v-=i;
  return *this;
}
T &T::operator*=( const unsigned int i )
{
//  kdDebug() << "T::*=" << endl;
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)*=i;
  v*=i;
  return *this;
}
T &T::operator/=( const unsigned int i )
{
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)/=i;
  v/=i;
  return *this;
} 

T &T::operator+=( const int i )
{
  if (isA("TArbitraryPrecision")) // Is this check really needed ? 
    // (it's virtual after all)
    return (*(TArbitraryPrecision *)this)+=i;
  v+=i;
  return *this;
}

T &T::operator-=( const int i )
{
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)-=i;
  v-=i;
  return *this;
}
T &T::operator*=( const int i )
{
//  kdDebug() << "T::*=" << endl;
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)*=i;
  v*=i;
  return *this;
}
T &T::operator/=( const int i )
{
  if (isA("TArbitraryPrecision"))
    return (*(TArbitraryPrecision *)this)/=i;
  v/=i;
  return *this;
}
T &T::abs(void)
{
  if (v<0) v*=-1;
  return *this;
}

T &T::sqrt(const double &idx)
{
  if (idx==2.0) 
  {
    if (v>0) v=::sqrt(v);
  }
  else
  {
    v=::pow(v,1.0/idx);
  }
    
  return *this;
}

T &T::sin( void )
{
  v=::sin(v);
  return *this;
}

T &T::cos( void )
{
  v=::cos(v);
  return *this;
}

T &T::tan( void )
{
  v=::tan(v);
  return *this;
}

T &T::exp( void )
{
  v=::exp(v);
  return *this;
}

T &T::log( void )
{
  v=::log(v);
  return *this;
}

T &T::log( const T &base )
{
  v=::log(v)/::log(base);
  return *this;
}

/*T T::operator-( const T &v1, const T &v2 )
{
  T tmp( v1 );
  tmp-=v2;
  return tmp;
}

T T::operator*( const T &v1, const T &v2 )
{
  T tmp( v1 );
  tmp*=v2;
  return tmp;
}

T T::operator/( const T &v1, const T &v2 )
{
  T tmp( v1 );
  tmp/=v2;
  return tmp;
}
*/
QString T::string(void) const
{
/*  char s[4935];
  sprintf(s,"%.4932Lg",v);
  return QString(s);
  */
  return QString::number((double)v,'g',16);
}

T::operator const double () const
{
  return v;
}

T &T::operator++( )
{ 
  ++v; 
  return *this; 
}

T &T::operator++( int )
{ 
  T *t=copy();
  Garbage::throwT(t);
  ++v; 
  return *t; 
}

T &T::operator--( )
{ 
  --v; 
  return *this; 
}

T &T::operator--( int )
{ 
  T *t=copy();
  Garbage::throwT(t);
  --v; 
  return *t; 
}

unsigned int T::m_decimalNumbers=32;

void T::setPrintDecimalNumbers(unsigned int i)
{
  T::m_decimalNumbers=i;
};

T *T::newT(const QString &s)
{
  if (T::TDataType==T_) return new T(s);
  return new TArbitraryPrecision(s);
}

T *T::newT(const double g)
{
  if (T::TDataType==T_) return new T(g);
  return new TArbitraryPrecision(g);
}

T *T::newT(const int g)
{
  if (T::TDataType==T_) return new T(g);
  return new TArbitraryPrecision(g);
}

T *T::newT(const unsigned int g)
{
  if (T::TDataType==T_) return new T(g);
  return new TArbitraryPrecision(g);
}

T *T::newT(const T &x)
{
//  if (T::TDataType==T_) return new T(x);
//  return new TArbitraryPrecision(x);
//  kdDebug() << QString("T::newT %1").arg(x.isA()) << endl;
//  kdDebug() << QString("newt %1").arg(x.string()) << endl;
  if (x.isA("T")) return new T(x);
//  kdDebug() << QString("newt TAP") << endl;
  return new TArbitraryPrecision((const TArbitraryPrecision &)x);
  
}

T &T::newTG(const double g)
{
  T *t=T::newT(g);
  Garbage::throwT(t);
  return *t;
}

T &T::newTG(const int g)
{
  T *t=T::newT(g);
  Garbage::throwT(t);
  return *t;
}

T &T::newTG(const unsigned int g)
{
  T *t=T::newT(g);
  Garbage::throwT(t);
  return *t;
}

void T::deleteT(T &x)
{
  delete &x;
}

void T::setDataType( TDataTypes t )
{
  T::TDataType=t;
}

T::TDataTypes T::TDataType=T_;
/*
T &operator+( const T &v1, const T &v2 )
{
  T *tmp=T::add(&v1,&v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator-( const T &v1, const T &v2 )
{
  T *tmp=T::sub(&v1,&v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator*( const T &v1, const T &v2 )
{
  T *tmp=T::mul(&v1,&v2);
  Garbage::throwT(tmp);
  return *tmp;
}

T &operator/( const T &v1, const T &v2 )
{
  T *tmp=T::div(&v1,&v2);
  Garbage::throwT(tmp);
  return *tmp;
}
*/
T *T::add(const T * v1, const T * v2)
{
  if ( v1->isA("TArbitraryPrecision") || v2->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::add((TArbitraryPrecision *)v1,(TArbitraryPrecision *)v2);
  T *tmp=new T( *v1 );
  *tmp+=*v2; 
  return tmp;
}

T *T::sub(const T * v1, const T * v2)
{
  if ( v1->isA("TArbitraryPrecision") || v2->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::sub((TArbitraryPrecision *)v1,(TArbitraryPrecision *)v2);
  T *tmp=new T( *v1 );
  *tmp-=*v2; 
  return tmp;
}

T *T::mul(const T * v1, const T * v2)
{
  if ( v1->isA("TArbitraryPrecision") || v2->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::mul((TArbitraryPrecision *)v1,(TArbitraryPrecision *)v2);
  T *tmp=new T( *v1 );
  *tmp*=*v2; 
  return tmp;
}

T *T::div(const T * v1, const T * v2)
{
  if ( v1->isA("TArbitraryPrecision") || v2->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::div((TArbitraryPrecision *)v1,(TArbitraryPrecision *)v2);
  T *tmp=new T( *v1 );
  *tmp/=*v2; 
  return tmp;
}

T *T::pow(const T * v1, const T * v2)
{
  if ( v1->isA("TArbitraryPrecision") || v2->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::pow((TArbitraryPrecision *)v1,(TArbitraryPrecision *)v2);
  T *tmp=new T( *v1 );
  // TODO
  return tmp;
}

T *T::add(const T * v1, const unsigned int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::add((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp+=i; 
  return tmp;
}

T *T::sub(const T * v1, const unsigned int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::sub((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp-=i; 
  return tmp;
}

T *T::mul(const T * v1, const unsigned int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::mul((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp*=i; 
  return tmp;
}

T *T::div(const T * v1, const unsigned int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::div((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp/=i; 
  return tmp;
}

T *T::add(const T * v1, const int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::add((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp+=i; 
  return tmp;
}

T *T::sub(const T * v1, const int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::sub((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp-=i; 
  return tmp;
}

T *T::mul(const T * v1, const int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::mul((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp*=i; 
  return tmp;
}

T *T::div(const T * v1, const int i)
{
  if ( v1->isA("TArbitraryPrecision") )
    return TArbitraryPrecision::div((TArbitraryPrecision *)v1,i);
  T *tmp=new T( *v1 );
  *tmp/=i; 
  return tmp;
}
